;+=----------------------------------------------------------------------------
; Name   : mrContainer
; RCS    : $Id: mrcontainer__define.pro,v 2.1 2004/11/10 10:38:33 olivert Exp $
; Methods:
;         <b>Constructor</b>
;         mrContainer::init            : constructor, no arguments
;         <br>
;         <b>Overloaded methods</b>
;         mrContainer::add             : sends notification events after adding
;         mrContainer::move            : sends notification event after moving
;         mrContainer::remove          : sends notification event after remove
;         <br>
;         <b>Notification</b>
;         mrContainer::notify          : see mrNotify::notify
;         mrContainer::enableNotify    : see mrNotify::enableNotify
;         mrContainer::disableNotify   : see mrNotify::disableNotify
;         <br>
;         <b>Private functions</b>
;         mrContainer__define          : the constructor
;         mrContainer::cleanup         : the destructor
; Desc  : An IDL_Container with added notification functionality upon
;         content change. Possible event types are 'add', 'remove' and 'move'.
;         Notifications are sent <b>after</b> the corresponding container method
;         has been executed. Notification methods are called without any
;         mrContainer-specific argument, i.e. only the object reference and the
;         event type are passed on.
; Code  : ct = obj_new('mrContainer)
;         ct->notify, myobj, 'notifyfunc', ['add,'remove']
;         ...
;         pro myobj::notifyfunc, sender, event
;         print, 'Container sent a ' + event + ' event
;         end
; Author: (c) Oliver Thilmann, oliver.thilmann@radfys.lu.se
; Licns : This program is published under the GPL. See gpl.txt for details
;=-----------------------------------------------------------------------------

;==============================================================================
;
;                          CONSTRUCTORS AND DESTRUCTORS
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mrContainer::init
; Syntax: obj = obj_new('mrContainer')
;         Notification is switched on by default.
;=-----------------------------------------------------------------------------
function mrContainer::init
	x = self->idl_container::init()
	y = self->mrNotify::init(['add','remove','move'])
	return, x and y
end

;+=----------------------------------------------------------------------------
; Name  : mrContainer::cleanup
; Desc  : Free allocated memory and call superclass destructor
;=-----------------------------------------------------------------------------
pro mrContainer::cleanup
	self->idl_container::cleanup
	self->mrNotify::cleanup
end

;==============================================================================
;
;                        OVERLOADED IDL_CONTAINER METHODS
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mrContainer::add
; Syntax: See IDL_Container
; Desc  : Calls IDL_Container::add and then notifies registered objects
;=-----------------------------------------------------------------------------
pro mrContainer::add, objects, _ref_extra=ex
	self->idl_container::add, objects, _strict_extra=ex
	self->sendNotification, 'add'
end

;+=----------------------------------------------------------------------------
; Name  : mrContainer::move
; Syntax: See IDL_Container
; Desc  : Calls IDL_Container::move and then notifies registered objects
;=-----------------------------------------------------------------------------
pro mrContainer::move, source, destination
	self->idl_container::move, source, destination
	self->sendNotification, 'move'
end

;+=----------------------------------------------------------------------------
; Name  : mrContainer::remove
; Syntax: See IDL_Container
; Desc  : Calls IDL_Container::remove and then notifies registered objects
;=-----------------------------------------------------------------------------
pro mrContainer::remove, child, _ref_extra=ex
	self->idl_container::remove, child, _strict_extra=ex
	self->sendNotification, 'remove'
end

;+=----------------------------------------------------------------------------
; Name : mrContainer__define
; Desc : The constructor.
;=-----------------------------------------------------------------------------
pro mrContainer__define
	obj = { MRCONTAINER,            $
			inherits mrNotify 		,$
			inherits IDL_Container	 $
	}
end