;+=----------------------------------------------------------------------------
; Name    : init_globals
; RCS     : $Id: init_globals.pro,v 1.1 2004/11/11 11:13:12 olivert Exp olivert $
; Desc    : Define and initialize the global structure
;           References to this global structure are stored:
;           - in the top widget_base's uvalue, to be used in the menu eventhandler
;           - in each lupemod
; Author  : (c) 2004 Oliver Thilmann, oliver.thilmann@radfys.lu.se
;=------------------------------------------------------------------------------
function init_globals

		; Structure holding the global variables.
		; This concerns the framework and the imgSet container via which all
		; inter-module communication should take place.

   g = { lupe_globals						,$
       version:             			''	,$ ; the program version/release
       version_date:                    ''  ,$ ; date of the release
       homedir:							''	,$ ; location of the .sav file
       workingdir:						''	,$ ; last used, i.e. working directory
       err:						 obj_new()	,$ ; the mrError object
       preferences:				 ptr_new()  ,$ ; pointer to the preferences structure
       command:                  obj_new()  ,$ ; hash table menu_label  -> module_name
       option:                   obj_new()  ,$ ; hash table menu_label  -> string option passed to module
       modules:					 obj_new()  ,$ ; hash table module_name -> module_instance
       i18n:                     obj_new()  ,$ ; hash table i18n_label  -> i18n_text
       top_modules:              obj_new()  ,$ ; container object holding currently mapped top mods
       emb_module:               obj_new()  ,$ ; a reference to the currently mapped embedded module
       topid:					 		0L	,$ ; widget id of the main widget_base
       imgSets:		 		     obj_new()	 $ ; a container for all image data (mrImageSet and mrImageSetDicom)
    }

		; Initialize global variables and create global objects

    cd, current   = wdir
    g.homedir     = wdir
    g.workingdir  = wdir
    g.err         = obj_new('mrerror', warnlimit=100, errorlimit=3) ; print to stdout for now
    g.top_modules = obj_new('IDL_Container')
    g.emb_module  = obj_new()

		; three hash tables for module handling

	g.command  = obj_new('mrhash', 'string', blocksize=40 )
	g.option   = obj_new('mrhash', 'string', blocksize=40 )
	g.modules  = obj_new('mrhash', 'object', blocksize=40 )

		; the central image container object

	g.imgSets     = obj_new('mrContainer')		; an idl_container with notification functionality

	gp = ptr_new( g, /no_copy )
	return, gp
end