;+=----------------------------------------------------------------------------
; Name    : mod_bbblc
; RCS     : $Id: mod_bbblc__define.pro,v 1.3 2004/11/25 09:05:25 olivert Exp olivert $
; Desc    : This module implementents a correction method for
;           extended blood-brain barrier leakage as described by Haselhorst
;           et al. in JMRI 11:495-505 (2000)
; Author  : (c) 2004 Oliver Thilmann, following an implementation by Ronnie Wirestam
;=------------------------------------------------------------------------------
function mod_bbblc::revision
	rev = '$Revision: 1.3 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::init
; Syntax: obj = obj_new('lupemod', type, exclusive )
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_bbblc::init, g

    res = self->lupemod::init( 'mod_bbblc', 'embedded', 0, g )
    if( res eq 0 ) then return, 0

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::readyToExecute
; Desc  : Checks whether an AIF is available - presence of an AIF should also
;         guarantee the presence of raw data.
;=-----------------------------------------------------------------------------
function mod_bbblc::readyToExecute, option
	i18n = (*self.g).i18n

	sigset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	ctrset = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt')
	baselineset = mrGetImageSet( (*self.g).imgSets, class='raw', name='baseline')
	if( sigset eq obj_new() or ctrset eq obj_new() or baselineset eq obj_new() ) then begin
		x = dialog_message(i18n->get('svd_missingRawImgSet'), title=i18n->get('Information'))
		return, 0
	end

	aifset = mrGetImageSet( (*self.g).imgsets, class='aif', name='global' )
	if( aifset eq obj_new() ) then begin
 		x = dialog_message(i18n->get('svd_noAIFDefined'), title=i18n->get('Information'))
		return, 0
	end
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::map
; Desc  : Use for 'top' and 'embedded' modules
;=-----------------------------------------------------------------------------
pro mod_bbblc::map, option

	i18n = (*self.g).i18n

		; create the base widget

	res = self->createBaseWidget( /row )
	if( res eq 0 ) then return

	l = widget_base( self.baseid, /column, /frame )
	r = widget_base( self.baseid, /column, /frame, /base_align_center )

		; left side: multidisplay & three result displays

	imgset = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
    nrows = 6 > ( imgset->nshots() + 9)/10


	l1 = widget_base( l, /row )
	self.mdisp = obj_new('mrmultidisplay', l1, 700, 420, 70, 70, 10, nrows )
	l1 = widget_base( l, /row )
	self.disp1 = obj_new('mrdisplay', l1, 232, 232, interpolate=0 )
	self.disp2 = obj_new('mrdisplay', l1, 232, 232, interpolate=0 )
	self.disp3 = obj_new('mrdisplay', l1, 232, 232, interpolate=0 )

		; right side: all the buttons I could think of (calculate)

    x  = widget_label( r, value=i18n->get('svd_bbblcTitle'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=250)
	r1 = widget_base ( r, xsize=230, /column )

		; image set names

	x  = widget_base ( r1, ysize=5 )
	r2 = widget_base ( r1, /row )
	x  = widget_label( r2, xsize=200, value=i18n->get('svd_resultImageSetNames') )
	r2 = widget_base ( r1, xsize=220, /column, /frame )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_CBVname') )
	self.idcbvname  = widget_text ( r3, xsize=130, value='rCBV BBBlc', /editable, uvalue={o:self, m:'dummyEvt'})
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_CBFname') )
	self.idcbfname  = widget_text ( r3, xsize=130, value='rCBF BBBlc', /editable, uvalue={o:self, m:'dummyEvt'} )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_MTTname') )
	self.idmttname  = widget_text ( r3, xsize=130, value='rMTT BBBlc', /editable, uvalue={o:self, m:'dummyEvt'} )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_T1name') )
	self.idt1name  = widget_text ( r3, xsize=130, value='T1', /editable, uvalue={o:self, m:'dummyEvt'} )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=100, value=i18n->get('svd_PERMname') )
	self.idpermname  = widget_text ( r3, xsize=130, value='Permeability', /editable, uvalue={o:self, m:'dummyEvt'} )

		; Display parameters

	x  = widget_base ( r1, ysize=5 )
	r2 = widget_base ( r1, /row )
	x  = widget_label( r2, xsize=200, value=i18n->get('svd_currentParameters') )

	r2 = widget_base ( r1, xsize=220, /column, /frame )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_signalValueThres') )
	self.idsigthres  = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_ctrValueThres') )
	self.idctrthres  = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_wMatrixCutoff') )
	self.idwcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_CBVcutoff') )
	self.idcbvcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_CBFcutoff') )
	self.idcbfcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )
	r3 = widget_base ( r2, /row )
	x  = widget_label( r3, xsize=150, value=i18n->get('svd_MTTcutoff') )
	self.idmttcutoff   = widget_label ( r3, xsize=50, value='-', /align_right )

		; T1 and permeability map, calculate button and progress

	x  = widget_base ( r1, ysize=5 )
	r2 = widget_base ( r1, /row )
	x  = widget_label( r2, xsize=200, value=i18n->get('svd_T1andPermMap') )
	r2 = widget_base( r1, /column, /frame, /base_align_center, /align_center, xsize=222 )
	self.disp4 = obj_new('mrdisplay', r2, 196, 196, 196, 196 )
	self.progressbar = obj_new( 'mrprogressbar', r1, xsize=222, /frame, title=i18n->get('CalculationProgress'), /remaining )
	x  = widget_button( r1, xsize=230, value=i18n->get('svd_calculate'), uvalue={o:self, m:'calcButtonEvt'})

		; close window button - no space for that one

	; x  = widget_button( r1, xsize=220, value=i18n->get('CloseWindow'), uvalue={o:self, m:'closeButtonEvt'} )

       ; set labels on the three result images

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'rCBV', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp1->labelView())->add, mdl

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'rCBF', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp2->labelView())->add, mdl

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'MTT', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp3->labelView())->add, mdl

    mdl = obj_new('IDLgrModel')
    txt = obj_new('IDLgrText', 'T1', locations=[.98,.02], color=[0,255,0], alignment=1.)
    mdl->add, txt
    (self.disp4->labelView())->add, mdl
    self.disp4Txt = txt

		; realize the widget and fill in the parameters

	self->placeAndRegister
	self->updateParameters

end

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::unmap
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_bbblc::unmap, _ref_extra=ex
	if( self.baseid eq 0L ) then return  ; uups - we don't exist :( Shouldn't happen

	obj_destroy, self.mdisp
	obj_destroy, self.disp1
	obj_destroy, self.disp2
	obj_destroy, self.disp3
	obj_destroy, self.disp4

	self->lupemod::unmap, _strict_extra=ex
end



;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::calcButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_bbblc::calcButtonEvt, evt
	self->calculate
end

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::closeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_bbblc::closeButtonEvt, evt
	self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::dummyEvt
; Desc  : catch events which are not used as widget fields are readout
;         directly when used
;=-----------------------------------------------------------------------------
pro mod_bbblc::dummyEvt, evt
end



;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::updateParameters
; Desc  : Get the parameters from preferences and display them
;=-----------------------------------------------------------------------------
pro mod_bbblc::updateParameters
	prefs = (*self.g).preferences

	widget_control, self.idsigthres,  set_value=string( (*prefs).ssvd_sigthres, format='(f6.1)' )
	widget_control, self.idctrthres,  set_value=string( (*prefs).ssvd_ctrthres, format='(f6.1)' )
	widget_control, self.idwcutoff,   set_value=string( fix( 100.*(*prefs).ssvd_matrixcutoff) )
	widget_control, self.idcbvcutoff, set_value=string( (*prefs).ssvd_cbvcutoff, format='(f5.0)' )
	widget_control, self.idcbfcutoff, set_value=string( (*prefs).ssvd_cbfcutoff, format='(f5.0)' )
	widget_control, self.idmttcutoff, set_value=string( (*prefs).ssvd_mttcutoff, format='(f5.0)' )
end

;+=----------------------------------------------------------------------------
; Name  : mod_bbblc::calculate
; Desc  : Perform the SVD calculation, slice by slice, and display the results
;         after each slice
;=-----------------------------------------------------------------------------
pro mod_bbblc::calculate

	i18n = (*self.g).i18n

	self->updateParameters
	self.progressbar->reset

	sigset      = mrGetImageSet( (*self.g).imgSets, class='raw', name='signal' )
	ctrset      = mrGetImageSet( (*self.g).imgSets, class='raw', name='ctr_xyt' )
	baselineset = mrGetImageSet( (*self.g).imgSets, class='raw', name='baseline')

	if( sigset eq obj_new() or ctrset eq obj_new() or baselineset eq obj_new() ) then begin
		x = dialog_message([i18n->get('svd_missingSets1'),i18n->get('svd_missingSets2')], /error, title=i18n->get('Error') )
		self->unmap
		return
	end

	nslices = sigset->nslices()
	xsize   = sigset->xsize()
	ysize   = sigset->ysize()

		; check whether the chosen name is already present in memory

	resultsets = mrGetImageSet( (*self.g).imgSets, class='result', count=nsets )
	widget_control, self.idcbvname, get_value=cbvname
	cbvname = strtrim(cbvname, 2)
	widget_control, self.idcbfname, get_value=cbfname
	cbfname = strtrim(cbfname, 2)
	widget_control, self.idmttname, get_value=mttname
	mttname = strtrim(mttname, 2)
	widget_control, self.idt1name, get_value=t1name
	t1name  = strtrim(t1name, 2)
	widget_control, self.idpermname, get_value=permname
	permname= strtrim(permname, 2)

	if( cbvname eq '' or cbfname eq '' or mttname eq '' or t1name eq '' or permname eq '' ) then begin
		x = dialog_message(i18n->get('svd_missingSetName'), title=i18n->get('Error'), /error)
		return
	end
	names = [cbvname, cbfname, mttname, t1name, permname]
	if( n_elements( uniq(names[sort(names)]) ) ne 5 ) then begin
		x = dialog_message(i18n->get('svd_imageSetNamesMustDiffer'), title=i18n->get('Error'), /error)
		return
	end
	if( nsets gt 0 and (*(*self.g).preferences).warnResultReplace eq 1 ) then begin
		setnames = resultSets[0]->name()
		for i = 1, nsets-1 do setnames = [setnames, resultSets[i]->name()]
		doubles = intersect( names, setnames )
		if( doubles[0] ne -1 ) then begin
			x = mrdialog_select( [i18n->get('svd_resultSetExistsWarn1'),doubles,i18n->get('svd_resultSetExistsWarn2'),$
								  i18n->get('svd_resultSetExistsWarn3')], ['Yes', 'No'], $
								  default=1, group_leader=self.baseid, bsize=80, title=i18n->get('Question'), /question )
			if( x eq 1 ) then return
		end
	end

		; create the result sets

	cbvset  = mrGetImageSet( (*self.g).imgSets, class='result', name=cbvname  )
	cbfset  = mrGetImageSet( (*self.g).imgSets, class='result', name=cbfname  )
	mttset  = mrGetImageSet( (*self.g).imgSets, class='result', name=mttname  )
	t1set   = mrGetImageSet( (*self.g).imgSets, class='result', name=t1name   )
	permset = mrGetImageSet( (*self.g).imgSets, class='result', name=permname )

	if( obj_valid(cbvset) ) then begin
		(*self.g).imgSets->remove, cbvset
		obj_destroy, cbvset
	end
	if( obj_valid(cbfset) ) then begin
		(*self.g).imgSets->remove, cbfset
		obj_destroy, cbfset
	end
	if( obj_valid(mttset) ) then begin
		(*self.g).imgSets->remove, mttset
		obj_destroy, mttset
	end
	if( obj_valid(t1set) ) then begin
		(*self.g).imgSets->remove, t1set
		obj_destroy, t1set
	end
	if( obj_valid(permset) ) then begin
		(*self.g).imgSets->remove, permset
		obj_destroy, permset
	end

	cbvset  = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=cbvname )
	cbfset  = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=cbfname )
	mttset  = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=mttname )
	t1set   = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=t1name  )
	permset = obj_new('mrimageset', nslices, xsize=xsize, ysize=ysize, class='result', name=permname)

		; get pointer to AIF

	aifset = mrGetImageSet( (*self.g).imgsets, class='aif', name='global' )
	if( aifset eq obj_new() ) then begin       ; just for safety, already checked in 'readyToExecute'
		x = dialog_message(i18n->get('svd_noAIFDefined'), title=i18n->get('Error'), /error)
		return
	end
	aif = (*(aifset->imgData()))[0]

		; get parameters from data - if the DICOM data contains no TR, we use the default value
		; without warning. The user has already been warned about that when calculating concentration
		; images

	dicom = (*(sigset->dicomData()))[0,0]
	tr    = dicom->getTagValue('acq_repetition_time')
	if( tr eq '' ) then tr = (*(*self.g).preferences).default_tr  ; take from defaults, if not found
	tr    = tr/1000.                                              ; convert time into seconds
	te    = dicom->getTagValue('acq_echo_time')
	if( te eq '' ) then te = (*(*self.g).preferences).default_te
	te    = te/1000.

		; perform the calculation

	imgdata = sigset->imgdata()
	for i = 0, nslices-1 do begin

		self.mdisp->label, /delete, /all
		for k = 0, sigset->nshots()-1 do begin
			row = k/10
			col = k mod 10
			self.mdisp->setImg, col, row, (*imgdata)[i, k], /no_contrastupdate
			self.mdisp->label, col, row, string( k, format='(i2)' )
		end
		self.mdisp->setContrast
		self.mdisp->display

		self->calcPerfusionImages, i, tr, te, aif, baselineset, sigset, ctrset, cbvset, cbfset, mttset, t1set, permset
		self.progressbar->update, (i+1.)/nslices
		self.disp1->setImg, (*(cbvset->imgdata()))[i]
		self.disp1->fitToScreen
		self.disp1->display
		self.disp2->setImg, (*(cbfset->imgdata()))[i]
		self.disp2->fitToScreen
		self.disp2->display
		self.disp3->setImg, (*(mttset->imgdata()))[i]
		self.disp3->fitToScreen
		self.disp3->display
	end

		; remember the ancestry

	cbvset->setParent,  sigset
	cbfset->setParent,  sigset
	mttset->setParent,  sigset
	t1set->setParent,   sigset
	permset->setParent, sigset

		; add the image sets to the central container

	(*self.g).imgSets->add, cbfset
	(*self.g).imgSets->add, cbvset
	(*self.g).imgSets->add, mttset
	(*self.g).imgSets->add, t1set
	(*self.g).imgSets->add, permset

end

;+=----------------------------------------------------------------------------
; Name   : mod_bbblc_fit
; Args   :
; Desc   : fit routine for mod_bbblc::calcPerfusionImages
;          Unluckily this has to go via a common block
;=-----------------------------------------------------------------------------
pro mod_bbblc_fit, time, par, y

    common mod_bbblc_fit, te, tr, t1, ctrint, ctrbar
    y = -1./te * alog( (1-exp(-tr*(1./t1+3.6*par[1]*ctrint))) / $
                       (1-exp(-tr/t1)) )  + par[0]*ctrbar+4.0*par[1]*ctrint
end

;+=----------------------------------------------------------------------------
; Name   : mod_bbblc::calcPerfusionImages
; Desc   : Calculates the perfusion images according to above mentioned paper.
;          Algorithm follows an implementation by Ronnie Wirestam.
;=-----------------------------------------------------------------------------
pro mod_bbblc::calcPerfusionImages, slice, tr, te, ai, baselineset, sigset, ctrset, cbvset, cbfset, mttset, t1set, permset

       ; common for the fitting routine, sigh.

    common mod_bbblc_fit, fitTE, fitTR, t1, ctrint, ctrbar

	aif = *ai

	nshots       = sigset->nshots()
	xsize        = sigset->xsize()
	ysize        = sigset->ysize()
	matrixcutoff = (*(*self.g).preferences).ssvd_matrixcutoff
	sigthres     = (*(*self.g).preferences).ssvd_sigthres
	bl           = baselineset->uvalue()  ; region over which average was calculated

       ; preparation of the arterial input function
       ; this is all as in mod_ssvd - see there for comments

    aif  = aif[(*bl)[1]+1:*]
    aif  = aif > 0
    aifc = aif

    maxval = max( aifc, maxid )                       ; cut away recirculation effects. Result is aifc
    for i = maxid+1, n_elements(aifc)-1 do begin
       if( aifc[i] le aifc[i-1] or aifc[i] gt .4*maxval ) then continue
       aifc[i:n_elements(aifc)-1] = 0.
       break
    end

       ; create the diagonal matrix for unfolding

    aifmatrix = fltarr( nshots-(*bl)[1]-1, nshots-(*bl)[1]-1 )
    for i = 0, nshots-(*bl)[1]-2 do begin
       for j = 0, nshots-(*bl)[1]-i-2 do begin
         aifmatrix[i+j, i] = aifc[j]
       end
    end

       ; define the result arrays for cbf, cbv, mtt

    cbf     = fltarr( xsize, ysize )
    cbv     = fltarr( xsize, ysize )
    mtt     = fltarr( xsize, ysize )
	t1map   = fltarr( xsize, ysize )
	permmap = fltarr( xsize, ysize )

		; one residue

    R=fltarr( nshots-(*bl)[1]-1 )

    	; decomposition and W matrix cut-off

    SVDC, aifmatrix, W, U, V, /double, /column
    wcut = matrixcutoff * max(abs(W))
    W    = (W ge wcut ) * W

		; now here's where the new stuff starts. Define some result arrays:
		; ctrbar = average signal over the whole slice
		; ctrint = integral of ctrbar

	ctrbar = 0.*fltarr( nshots-(*bl)[1]-1 ) ; no, I don't remember why the >> 0.* <<
	ctrint =    fltarr( nshots-(*bl)[1]-1 )

	time   = (*bl)[1]+1 * tr + tr * findgen( nshots-(*bl)[1]-1 )

		; create the t1-map and calc average for overflow values/
		; the first signal values of the time sequence will be slightly higher as
		; equilibrium has not been reached yet. From the first data value and the
		; equilibrium value, T1 can be easily calculated.
		; As tr is in seconds, t1map is in seconds as well!

    s0     = *(*(baselineset->imgdata()))[slice]   ; the baseline image
    sig0   = *(*(sigset->imgData()))[slice,0]      ; signal image @ T=0
    ctr    = (*(ctrset->imgdata()))[slice]

	for x = 0, xsize-1 do begin
		for y = 0, ysize-1 do begin
			if( s0[x,y] gt sigthres ) then begin
				if( s0[x,y] lt sig0[x,y] ) then                             $
					t1map[x,y] = -tr/alog( 1-( s0[x,y]/sig0[x,y] ))    $
				else                                                        $
					t1map[x,y] = 4.0
			end else begin
				t1map[x,y] = 0.
			end
		end
	end

		; remove overflow and underflow values from the t1 map
		; this step is obviously a bit unclean in case we have two adjacent overflow values
		; besides it could be done more elegantly in IDL, but what the heck

	for x = 1, xsize-2 do begin
		for y = 1, ysize-2 do begin
			if( t1map[x,y] ge 4. ) then t1map[x,y] = (t1map[x-1,y]+t1map[x+1,y]+t1map[x,y-1]+t1map[x,y+1])/4.
		end
	end
	t1map = 0.001 > temporary(t1map) < 4.

       ;
       ; display the obtained t1 map
       ;

	self.disp4->setImg, ptr_new(t1map), /no_copy
	self.disp4->fitToScreen
	self.disp4Txt->setProperty, strings='T1'
	self.disp4->display

		; calculate average over the whole image - the average vector starts at index
		; '0' which corresponds to image (*bl)[1]+1 !

	for i = 0, nshots-(*bl)[1]-2 do begin
		nvals   = 0.
		shotno = (*bl)[1]+1+i
		ctrbar[i] = 0.
		for x = 0, xsize-1 do begin
			for y = 0, ysize-1 do begin
				if( s0[x,y] gt sigthres ) then begin
					ctrbar[i] = ctrbar[i] + (*ctr)[x,y,shotno]
					nvals = nvals+1.
				end
			end
		end
		ctrbar[i] = ctrbar[i]/nvals
	end

       ; calculate integral over ctrbar

	ctrbar = ctrbar - min(ctrbar)              ; shift average such that minimum is '0'
	ctrint[0] = ctrbar[0]
	ctrint[1] = ctrbar[0]+ctrbar[1]*tr
	ctrint[2] = ctrbar[0]+ctrbar[1]*tr + ctrbar[2]*tr
	for i = 3, nshots-(*bl)[1]-2 do begin
		ctrint[i] = int_tabulated( time[0:i], ctrbar[0:i] )
	end

       ; prepare the fitting

    weights    = fltarr( nshots-(*bl)[1]-1 )
    weights[*] = 1.


		; now do the calculation for each image pixel for this slice
		; The loop itself consumes about 6 seconds of time on my machine (as compared
		; to 10 seconds for the svsol) - 3 seconds of which are needed to fill 'signal'.
		; It actually makes no difference whether signal is defined (signal[*] = fltarr(something))
		; or newly created in each loop - the latter is even a little bit faster.
		; another 1.5 seconds are spend on the s0[x,y] gt sigthres comparison


	signal = fltarr( nshots-(*bl)[1]-1 )
	totaif = total(aif)
	fitTE  = te
	fitTR  = tr

	self.disp4Txt->setProperty, strings=(*self.g).i18n->get('Permeability')

;	for x = 50, 70 do begin             ; testing
;		for y = 50, 70 do begin
	for x = 0, xsize-1 do begin
		for y = 0, ysize-1 do begin
			signal[*] = (*ctr)[x,y,(*bl)[1]+1:nshots-1]
			if( s0[x,y] gt sigthres and total(signal) gt 0. ) then begin

				; fit the curve

				chisqr  = fltarr( 10 )
				pars    = fltarr( 10, 2 )
				par     = fltarr( 2 )
				t1      = t1map[x,y]
				!Quiet  = 1
				for i = 0, 9 do begin
					pars[i,0] = 0.02
					pars[i,1] = (-0.04)+(1./100.*i)
					par[0]    = pars[i,0]
					par[1]    = pars[i,1]
					fit       = curvefit(time, signal, weights, par, chisq=chi, $
					function_name='mod_bbblc_fit', itmax=999, /noderivative, tol=0.001)
					chisqr[i] = chi
				end
				chimin = min( chisqr, idx, /NaN )
				par[0] = pars[idx,0]
				par[1] = pars[idx,1]
				fit = curvefit( time, signal, weights, par, chisq=sigma, $
				function_name='mod_bbblc_fit', itmax=999, /noderivative, tol=0.001)
				if( not finite(sigma) ) then sigma=100.

				corrsig = signal - 4.*par[1]*ctrint + te * $
							alog(                                               $
									(1 - exp( -tr*(1./t1+3.6*par[1]*ctrint))) / $
									(1 - exp( -tr/t1 ) )                        $
							)

				R[*] = svsol( u, w, v, corrsig, /double, /column )

				cbf[x,y] = 6000 * 0.705 / tr * max(R[*]) * total(corrsig) / ( total(aif) * total(R[*]) )
				cbv[x,y] = 0.705 * 100. * total(corrsig) / total(aif)
				mtt[x,y] = total(R[*]) * tr / max(R[*])

				; alternatively one could replace 'total' by 'int_tabulated(time, variable), but
				; then it's harder to compare results from uncorrected and corrected methods
				permmap[x,y] = 1000. * par[1]
			end

			; quick hack to repair things when tumor fitting went wrong
			; -> do conventional calculation then
			; signal correction can nevertheless be quite severe, not because
			; of a corrected signal, but corrected recirculation effects.
			; So the conventionally calculated values will differ quite a bit
			; from the corrected values

		end

		self.disp4->setImg, ptr_new(permmap), /no_copy
		self.disp4->fitToScreen
		self.disp4->display

	end

       ; cut off extreme values

    cbv = cbv < (*(*self.g).preferences).ssvd_cbvcutoff
    cbv = cbv > 0.
    cbf = cbf < (*(*self.g).preferences).ssvd_cbfcutoff
    cbf = cbf > 0.
    mtt = mtt < (*(*self.g).preferences).ssvd_mttcutoff
    mtt = mtt > 0.
    permmap = permmap > 0.

		; store the result in the resultsets

	ptr_free, (*(cbvset->imgdata()))[slice]
	ptr_free, (*(cbfset->imgdata()))[slice]
	ptr_free, (*(mttset->imgdata()))[slice]
	ptr_free, (*(t1set->imgData()))[slice]
	ptr_free, (*(permset->imgData()))[slice]

	(*(cbvset->imgdata()))[slice]  = ptr_new( cbv, /no_copy )
	(*(cbfset->imgdata()))[slice]  = ptr_new( cbf, /no_copy )
	(*(mttset->imgdata()))[slice]  = ptr_new( mtt, /no_copy )
	(*(t1set->imgdata()))[slice]   = ptr_new( t1map, /no_copy )
	(*(permset->imgdata()))[slice] = ptr_new( permmap, /no_copy )

end


;+=----------------------------------------------------------------------------
; Name  : mod_bbblc__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_bbblc__define
	obj = { MOD_BBBLC		   			,$
			inherits LUPEMOD 	  		,$
			mdisp:			 obj_new()	,$ ; multi display for the full image set of one slice
			disp1:			 obj_new()	,$ ; rCBV result
			disp2:			 obj_new()	,$ ; rCBF result
			disp3:			 obj_new()	,$ ; MTT  result
			disp4:           obj_new()  ,$ ; T1 map and the permeability maps
			disp4Txt:		 obj_new()  ,$ ; the label in display 4 changes T1/Permeability
			progressbar:	 obj_new()	,$
			idcbvname:				0L	,$
			idcbfname:				0L	,$
			idmttname:				0L	,$
			idt1name:				0L	,$ ;
			idpermname:				0L	,$ ; permeability map name
			idsigthres:				0L	,$
			idctrthres:				0L	,$
			idwcutoff:				0L	,$
			idcbvcutoff:			0L	,$
			idcbfcutoff:			0L	,$
			idmttcutoff:			0L	 $
		  }
end

