;+=----------------------------------------------------------------------------
; Name    : mod_filebrowser
; RCS     : $Id: mod_filebrowser__define.pro,v 1.2 2004/11/25 09:07:02 olivert Exp olivert $
; Desc    : A simple file-browser
; Author  : (c) 2004 Oliver Thilmann
; TODO    :
;=------------------------------------------------------------------------------
function mod_filebrowser::revision
	rev = '$Revision: 1.2 $
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::init
; Desc  :
;=-----------------------------------------------------------------------------
function mod_filebrowser::init, g

    res = self->lupemod::init( 'mod_filebrowser', 'top', 0, g )    ; not exclusive
    if( res eq 0 ) then return, 0

	self.dispdicom   = 0
	self.interpolate = 0
	self.rotate      = 1

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::readyToExecute
; Desc  :
;=-----------------------------------------------------------------------------
function mod_filebrowser::readyToExecute, option
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::map
; Desc  : Wir verwirklichen uns selbst
;=-----------------------------------------------------------------------------
pro mod_filebrowser::map, option

	i18n = (*self.g).i18n

		; create the base widget

	res = self->createBaseWidget( /row, scr_xsize=820, scr_ysize=635, title=i18n->get('fb_title') )
	if( res eq 0 ) then return

		; create UI

	b1 = widget_base( self.baseid, /column, /frame )  ; left : mrdisplay & Roi UI
	b2 = widget_base( self.baseid, /column, /frame )  ; right: all the rest

		; b1: mrdisplay and sliders

	self.roi       = obj_new('mrroiui',    b1, row=1, /frame, xsize=512 )
	self.display   = obj_new('mrdisplay',  b1, 512, 512, 512, 512, interpolate=self.interpolate )
	self.contrast  = obj_new('mrcontrast', b1, colbars=0, xsize=512, ysize=50 )

		; b2: title, buttons, contrast - sometimes within separate base widgets

    x  = widget_label( b2, value=i18n->get('fb_title'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=260)
	c2 = widget_base( b2, xsize=250, /align_center, /column )
	x  = widget_label( c2, value='', ysize=10 )

	x  = mrwidget_fileselect( c2, /frame, /no_cancel, scr_xsize=250, scr_ysize=370, filter=['*','*.dcm'], uvalue={o:self, m:'fileSelectEvt'} )

	x  = widget_label( c2, value='', ysize=13 )
	x  = widget_label ( c2, value=i18n->get('Display')+':', /align_left )
	r  = widget_base  ( c2, /column, /frame )
	r1 = widget_base  ( r , /row, ysize=30 )
	x  = widget_label ( r1, value=i18n->get('fb_displayDicomInfo'), xsize=200 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	self.iddicom = widget_button( r2, value='', uvalue={o:self, m:'dicomButtonEvt'} )
	widget_control, self.iddicom, set_button=self.dispdicom

	r1 = widget_base  ( r , /row )
	x  = widget_label ( r1, value=i18n->get('fb_interpolateImage'), xsize=200 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	self.idinter = widget_button( r2, value='', uvalue={o:self, m:'interpolateButtonEvt'} )
	widget_control, self.idinter, set_button=self.interpolate

	r1 = widget_base  ( r , /row )
	x  = widget_label ( r1, value=i18n->get('fb_rotateImage'), xsize=200 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	self.idrotate = widget_button( r2, value='', uvalue={o:self, m:'rotateButtonEvt'} )
	widget_control, self.idrotate, set_button=self.rotate


	x  = widget_label( c2, value='', ysize=13 )
	x  = widget_button( c2, value=i18n->get('CloseWindow'), xsize=186, /frame, uvalue={o:self, m:'closeButtonEvt'} )

		; realize the widget and register with the xmanager

	self->placeAndRegister

		; now for the remaining objects

	self.zoom = obj_new('mrzoom')
	self.zoom->selfregister, self.display, button=2, /border
	self.pan  = obj_new('mrpan')
	self.pan->selfregister, self.display, button=2, /noborder
	self.cursorInfo = obj_new('mrCursorInfo')
	self.cursorInfo->selfregister, self.display, toggleKey='c'
	self.dataflash = obj_new('mrdataflash')
	self.dataflash->selfregister, self.display
	self.contrast->selfregister, self.display, button=3, /bordermode

		; activate the image set selector and create initial 'imgsetChange' event


end

;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::unmap
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_filebrowser::unmap, _ref_extra=ex

	if( self.baseid eq 0L ) then return  ; uups - we don't exist :( Shouldn't happen

	obj_destroy, self.display
	obj_destroy, self.dataflash
	obj_destroy, self.zoom
	obj_destroy, self.contrast
	obj_destroy, self.cursorinfo
	obj_destroy, self.pan

		; remove from notification lists
		; we also registered for notification from the currenly displayed image set
		; in case of palette changes via the coloring module

	self->lupemod::unmap, _strict_extra=ex
end



;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::dicomButtonEvt
; Desc  : This event can only occur when a dicom file is on display
;=-----------------------------------------------------------------------------
pro mod_filebrowser::dicomButtonEvt, evt
	self.dispDicom = evt.select
	if( self.dispDicom ) then self.dataflash->show else self.dataflash->hide
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::interpolateButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_filebrowser::interpolateButtonEvt, evt
	self.interpolate = evt.select
	self.display->setInterpolate, self.interpolate
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::rotateButtonEvt
; Desc  : This event can only occur if an image is on display
;=-----------------------------------------------------------------------------
pro mod_filebrowser::rotateButtonEvt, evt
	self.rotate = evt.select
	if( self.rotate ) then img = rotate( *self.img, 7 ) else img=( *self.img )
	self.display->setImg, ptr_new(img, /no_copy), /no_copy, /update
	self.display->display
end


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::fileSelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_filebrowser::fileSelectEvt, evt
	file = evt.path+evt.file
	if( obj_valid( self.dicom ) ) then obj_destroy, self.dicom
	if( ptr_valid( self.img   ) ) then ptr_free, self.img

		; try to read as dicom file

	self.dicom = obj_new('mrdicom')
	res = self.dicom->read( file )
	if( res eq 1 ) then begin

		self.img = self.dicom->img()
		self.dataflash->setDicomSource, self.dicom

	end else begin

			; destroy dicom related stuff

		obj_destroy, self.dicom
		self.dicom = obj_new()
		self.img   = ptr_new()

			; here we could try to read other file types

	end

	self->display
end


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::closeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_filebrowser::closeButtonEvt, evt
	self->unmap
end



;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser::display
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_filebrowser::display

		; kill label, if present

	view = self.display->labelView()
	model = view->getByName('lupe_filebrowser')
	if( obj_valid(model) ) then begin
		view->remove, model
		obj_destroy, model
	end

		; if no dicom file present, display 'cannot display' message

	if( obj_valid( self.dicom ) ) then begin

		widget_control, self.iddicom, /sensitive
		widget_control, self.idinter, /sensitive
		widget_control, self.idrotate, /sensitive
		if( self.dispdicom ) then self.dataflash->show else self.dataflash->hide

	end else begin

			; no DICOM

		widget_control, self.iddicom, sensitive=0
		self.dataflash->hide

	end

	if( ptr_valid( self.img ) ) then begin

		self.roi->unregister
		if( self.rotate ) then img = rotate( *self.img, 7 ) else img = *self.img
		self.display->setImg, ptr_new(img, /no_copy), /no_copy
		self.display->fitToScreen
		self.display->activate
		widget_control, self.idrotate, /sensitive
		widget_control, self.idinter, /sensitive
		self.roi->selfregister, self.display, button=1

	end else begin

		; no image for display

		self.roi->unregister
		widget_control, self.idinter, sensitive=0
		widget_control, self.idrotate, sensitive=0
		self.display->clear
		model = obj_new('IDLgrModel', name='lupe_filebrowser')
		text  = obj_new('IDLgrText', color=[0,255,0], alignment=0.5, locations=[.5,.5], strings='File can not be displayed' )
		model->add, text
		view->add, model
		self.display->deactivate

	end

	self.display->display

end


;+=----------------------------------------------------------------------------
; Name  : mod_filebrowser__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_filebrowser__define
	obj = { mod_filebrowser   		   ,$
			inherits LUPEMOD   		   ,$
			roi:			 obj_new() ,$
			display:		 obj_new() ,$
			contrast:		 obj_new() ,$
			zoom:			 obj_new() ,$
			cursorinfo:		 obj_new() ,$
			pan:			 obj_new() ,$
			dataflash:		 obj_new() ,$
			dicom:			 obj_new() ,$
			img:			 ptr_new() ,$
			iddicom:				0L ,$
			idinter:				0L ,$
			idrotate:				0L ,$
			dispdicom:				 0 ,$
			interpolate:			 0 ,$
			rotate:					 0  $
		  }
end

