;+=----------------------------------------------------------------------------
; Name    : mod_imagebrowser
; RCS     : $Id: mod_imagebrowser__define.pro,v 1.3 2004/11/25 08:59:02 olivert Exp olivert $
; Desc    :
;           Let's the user browse all image sets currently available in memory,
;           that is all image sets that contain two-dimensional images. The others
;           are listed, but cannot be displayed.
;           Changes in the central image set container are automatically tracked
;           via widget_selectImgSet.
;           We also check for palette changes within one image set. Image data
;           changes however are not recognized (lame excuse: some of the modules
;           work around the set/get functions of mrImageSet anyway by directly
;           writing into the matrix).
;           The ROI analysis window could be a widget_text instead of a widget_list.
;           The latter looks much nicer, but copying text from a list is not possible.
;           For now looks go ahead of functionality :) This can be changed easily, if
;           needed.
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_imagebrowser::revision
	rev = '$Revision: 1.3 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::init
; Syntax: obj = obj_new('lupemod', type, exclusive )
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_imagebrowser::init, g

    res = self->lupemod::init( 'mod_imagebrowser', 'top', 0, g )    ; not exclusive
    if( res eq 0 ) then return, 0

	self.imgSet   = obj_new()

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::readyToExecute
; Syntax:
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_imagebrowser::readyToExecute, option
	i18n = (*self.g).i18n

	if( (*self.g).imgSets->count() eq 0 ) then begin
		x = dialog_message( i18n->get('err_noImageSetsInMemory'), title=i18n->get('Info'), /info)
		return, 0
	end
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::map
; Desc  : Wir verwirklichen uns selbst
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::map, option

	i18n = (*self.g).i18n

		; create the base widget

	res = self->createBaseWidget( /row, scr_xsize=800, scr_ysize=720, title=i18n->get('ib_windowTitle'))
	if( res eq 0 ) then return

		; create UI

	b1 = widget_base( self.baseid, /column, /frame )  ; left : mrdisplay & sliders
	b2 = widget_base( self.baseid, /column, /frame )  ; right: title/buttons/contrast

		; b1: mrdisplay and sliders

	r1 = widget_base( b1, /row )
	self.roiui      = obj_new('mrroiui', r1, row=1, /frame, xsize=512 )

	r1 = widget_base( b1, /row )
	self.display   = obj_new('mrdisplay', r1, 512, 512, 512, 512, interpolate=0 )
	self.idvslider = widget_slider( r1, xsize=30, ysize=512, /vertical, value=0, sensitive=0, uvalue={o:self, m:'vSliderEvt'} )
	r2 = widget_base( b1, /column )
	self.idhslider = widget_slider( r2, xsize=512, ysize=30, value=0, sensitive=0, uvalue={o:self, m:'hSliderEvt'} )
	self.contrast = obj_new('mrcontrast', r2, xsize=512, ysize=83, colbars=2 )


		; b2: title, buttons, contrast - sometimes within separate base widgets

    x  = widget_label( b2, value=i18n->get('ib_title'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=210)

	c2 = widget_base( b2, xsize=200, /align_center, /column )

	x  = widget_label( c2, value='', ysize=10 )

	x  = widget_label   ( c2, value=i18n->get('ImageSet')+':', /align_left)
	r  = widget_base    ( c2, /column, /frame )
	self.imgsetSelector = obj_new('widget_selectImgSet', r, self.g, classTitle=i18n->get('Class')+':', nameTitle=i18n->get('Name')+':', titleSize=60, listSize=120 )
	r1 = widget_base    ( r, /column, xsize=186 )
	x  = widget_button  ( r1, value=i18n->get('ib_deleteThisImageSet'), xsize=180, /align_center, uvalue={o:self, m:'deleteImgSetEvt'} )

	x  = widget_label( c2, value='', ysize=19 )

	x  = widget_label ( c2, value=i18n->get('Display')+':', /align_left )
	r  = widget_base  ( c2, /column, /frame )
	r1 = widget_base  ( r , /row, ysize=30 )
	x  = widget_label ( r1, value=i18n->get('ib_displayDicomInfo'), xsize=150 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	self.iddicom = widget_button( r2, value='', uvalue={o:self, m:'dicomButtonEvt'} )
	widget_control, self.iddicom, set_button=self.dispdicom

	r1 = widget_base  ( r , /row )
	x  = widget_label ( r1, value=i18n->get('ib_interpolateImage'), xsize=150 )
	r2 = widget_base  ( r1, /row, /nonexclusive )
	self.idinter = widget_button( r2, value='', uvalue={o:self, m:'interpolateButtonEvt'} )

	x  = widget_label( c2, value='', ysize=19 )

	x  = widget_label ( c2, value='Roi analysis:', /align_left )
	r  = widget_base  ( c2, /column, /frame )
	r1 = widget_base  ( r , /row, ysize=25)
	x  = widget_button( r1, value='Store', scr_xsize=90, uvalue={o:self, m:'storeRoiEvt'} )
	x  = widget_button( r1, value='Recall', scr_xsize=90, uvalue={o:self, m:'recallRoiEvt'} )
	x  = widget_list  ( r , scr_xsize=186, scr_ysize=298, /frame, value='', font='fixedsys', uvalue={o:self, m:'dummyEvt'} )
	self.idlist = x

		; scheusslich, scheusslich

	x  = widget_label( c2, value='', ysize=15 )
	x  = widget_button( c2, value=i18n->get('CloseWindow'), xsize=186, /frame, uvalue={o:self, m:'closeButtonEvt'} )

		; realize the widget and register with the xmanager

	self->placeAndRegister

		; now for the remaining objects

	self.zoom = obj_new('mrzoom')
	self.zoom->selfregister, self.display, button=2, /border
	self.pan  = obj_new('mrpan')
	self.pan->selfregister, self.display, button=2, /noborder
	self.cursorInfo = obj_new('mrCursorInfo')
	self.cursorInfo->selfregister, self.display, toggleKey='c'
	self.dataflash = obj_new('mrdataflash')
	self.dataflash->selfregister, self.display
	self.contrast->selfregister, self.display, button=3, /bordermode

		; get keypress events from the display object

	self.display->registerEvt, self, 'KeyPress'
	self.roiui->notify, self, 'roiEvtHandler', ['modify','change']

		; activate the image set selector and create initial 'imgsetChange' event

	self.imgsetSelector->notify, self, 'imgsetChangeEvt', 'change'
	self->imgSetChange

end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::unmap
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::unmap, _ref_extra=ex

	if( self.baseid eq 0L ) then return  ; uups - we don't exist :( Shouldn't happen

	obj_destroy, self.display
	obj_destroy, self.dataflash
	obj_destroy, self.zoom
	obj_destroy, self.contrast
	obj_destroy, self.cursorinfo
	obj_destroy, self.pan
	obj_destroy, self.imgsetSelector
	obj_destroy, self.roiui

		; remove from notification lists
		; we also registered for notification from the currenly displayed image set
		; in case of palette changes via the coloring module

	if( self.imgSet ne obj_new() ) then self.imgSet->notify, self, /remove
	self->lupemod::unmap, _strict_extra=ex
end



;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::VSliderEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::VSliderEvt, evt
	self.currSlice = evt.value
	self->display, /update
	self->evalROIs
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::HSliderEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::HSliderEvt, evt
	self.currShot = evt.value
	self->display, /update
	self->evalROIs
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::dicomButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::dicomButtonEvt, evt
	self.dispDicom = evt.select
	self->display, /update
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::interpolateButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::interpolateButtonEvt, evt
	self.display->setInterpolate, evt.select
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::deleteImgSetEvt, evt
; Desc  : Deletes the currently displayed image set from memory without
;         much further ado. All other update happens automatically via
;         callback from the imgSets mrContainer object
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::deleteImgSetEvt, evt
	class = self.imgsetSelector->class()
	name  = self.imgsetSelector->name()
	if( class eq '' or name eq '' ) then return
	imgSet = mrGetImageSet( (*self.g).imgSets, class=class, name=name, count=nsets )
	if( nsets eq 0 ) then return ; shouldn't happen
	allSets = mrGetImageSet( (*self.g).imgSets, count=asets )
	for i = 0, asets-1 do begin
		if( allSets[i]->parent() eq imgSet ) then allSets[i]->setParent, obj_new()
	end
	(*self.g).imgSets->remove, imgSet
	obj_destroy, imgSet
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::storeRoiEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::storeRoiEvt, evt
	roigroup = self.roiui->roigroup()
	if( roigroup eq obj_new() ) then return
	if( obj_valid( self.roigroup ) ) then obj_destroy, self.roigroup
	self.roigroup = roigroup->duplicate()
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::recallRoiEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::recallRoiEvt, evt
	if( self.roigroup eq obj_new() ) then begin
		x = dialog_message('No ROIs stored', /info )
		return
	end
	ngroup = self.roigroup->duplicate()
	self.roiui->setRoigroup, ngroup
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::closeButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::closeButtonEvt, evt
	self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::dummyEvt
; Desc  : event catcher
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::dummyEvt, evt
end

;==============================================================================
;
;                                C A L L   B A C K
;
;           entrance points for the call back and notification methods
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::imgSetChangeEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::imgSetChangeEvt, sender, evttype, evt
	self->imgsetChange
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::mrDisplayEvtHandler, evt
; Desc  : key press events from the mrDisplay object
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::mrDisplayEvtHandler, evt
	if( self.imgset eq obj_new() ) then return
	dims = self.imgset->size()
	case (*evt).key of
		5: begin													; left
			if( dims[0] eq 1 ) then return
			self.currShot = (self.currShot-1) > 0
			widget_control, self.idhslider, set_value=self.currShot
			self->display, /update
		end
		6: begin													; right
			if( dims[0] eq 1 ) then return
			self.currShot = (self.currShot+1) < (dims[2]-1)
			widget_control, self.idhslider, set_value=self.currShot
			self->display, /update
		end
		7: begin													; up
			self.currSlice = (self.currSlice+1) < (dims[1]-1)
			widget_control, self.idvslider, set_value=self.currSlice
			self->display, /update
		end
		8: begin													; down
			self.currSlice = (self.currSlice-1) > 0
			widget_control, self.idvslider, set_value=self.currSlice
			self->display, /update
		end
		else: ; do nothing
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::imgSetPaletteChange
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::imgSetPaletteChange, sender, event
	if( sender ne self.imgSet ) then x = dialog_message('illegal message sender')
	self.display->setPalette, self.imgSet->Palette()
	self.display->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::roiEvtHandler, sender, evttype, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::roiEvtHandler, sender, evttype, evt
	self->evalRois
end


;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::imgSetChange
; Desc  : The image set to display has been changed. This might have either
;         occurred because the user has selected a new set to display or
;         due to an update event - which might have caused the current image
;         set to disappear.
;         Be aware that self.currentName might equal ''
;         This method also does all button enabling/disabling
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::imgSetChange

	i18n = (*self.g).i18n

	self.roiui->unregister
	if( obj_valid(self.imgSet) ) then self.imgSet->notify, self, /remove  ; no notifications

		; kill the labelview, if it is present

	view = self.display->labelView()
	model = view->getByName('lupe_imagebrowser')
	if( obj_valid(model) ) then begin
		view->remove, model
		obj_destroy, model
	end

		; it may happen, that the user creates an image set with empty content
		; (i.e. NULL pointers only) - generally, this is a design fault, but we
		; catch it here nevertheless
		; Actually with newer modules, it may happen that image sets are only
		; partially filled.
		; !!! Need some coding here !!!

	nsets       = (*self.g).imgSets->count()
	class       = self.imgsetSelector->class()
	name        = self.imgsetSelector->name()
	self.imgset = mrGetImageSet( (*self.g).imgSets, class=class, name=name )
	if( obj_valid(self.imgset) ) then begin
		if( (*(self.imgset->imgData()))[0] ne ptr_new() ) then ndims = size( *(*(self.imgset->imgData()))[0], /n_dimensions ) else ndims=0
	end else begin
		self.imgset = obj_new()
		ndims=0
	end

		; we can only display 2-dim images - display a 'cannot display'
		; message for other sets (e.g. global AIF or xyt images)

	if( ndims ne 2 ) then begin
		widget_control, self.idvSlider,  sensitive=0
		widget_control, self.idhSlider,  sensitive=0
		widget_control, self.iddicom,    sensitive=0
		widget_control, self.idinter,    sensitive=0
		self.display->clear
		model = obj_new('IDLgrModel', name='lupe_imagebrowser')
		if( nsets eq 0 ) then $
			text  = obj_new('IDLgrText', color=[0,255,0], alignment=0.5, locations=[.5,.5], strings=i18n->get('ib_noImageSetsAvailable')) $
		else $
			text  = obj_new('IDLgrText', color=[0,255,0], alignment=0.5, locations=[.5,.5], strings=i18n->get('ib_setCannotBeDisplayed'))
		model->add, text
		view->add, model
		self.dataflash->hide
		self.display->deactivate
		self.display->display
		self->evalROIs
		return
	end
		; find out about image set dimensions and adapt the sliders

	dims = self.imgset->size()
	case dims[0] of
		0: begin
			widget_control, self.idvslider, sensitive=0, set_slider_min=0, set_slider_max=1, set_value=0
			widget_control, self.idhslider, sensitive=0, set_slider_min=0, set_slider_max=1, set_value=0
			self.currSlice = 0
			self.currShot = 0
		end
		1: begin
			self.currSlice = self.currSlice < (dims[1]-1)
			self.currShot  = 0
			widget_control, self.idvslider, sensitive=1, set_slider_max=dims[1]-1, set_value=self.currSlice
			widget_control, self.idhslider, sensitive=0, set_slider_max=0,          set_value=0
		end
		2: begin
			self.currSlice = self.currSlice < (dims[1]-1)
			self.currShot  = self.currShot  < (dims[2]-1)
			widget_control, self.idvslider, sensitive=1, set_slider_max=dims[1]-1, set_value=self.currSlice
			widget_control, self.idhslider, sensitive=1, set_slider_max=dims[2]-1, set_value=self.currShot
		end
	end

		; is that a dicom set? Enable/disable dicom button accordingly
		; If one of the ancestry is a dicom set, set dataflash font color to red

	parentSet = findDicomParent( self.imgset )
	case parentSet of
		self.imgSet: begin
			widget_control, self.iddicom, sensitive=1
			self.dataflash->setColor, [0,255,0]
		end
		obj_new() : begin
			widget_control, self.iddicom, sensitive=0
			self.dataflash->hide
		end
		else: begin
			widget_control, self.iddicom, sensitive=1
			self.dataflash->setColor, [255,0,0]
		end
	end

	widget_control, self.idinter,    sensitive=1

		; display the new image

	self.display->setPalette, self.imgset->palette()
	self->display
	self.display->activate
	self.roiui->selfregister, self.display, button=1
	self->evalROIs
	self.imgSet->notify, self, 'imgSetPaletteChange', 'palettechange'
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::display
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::display, update=update

	widget_control, self.idvslider, get_value=slice
	widget_control, self.idhslider, get_value=shot

	matrixSize = self.imgset->size( /n_dimensions )
	case matrixSize of
		0: img = (*(self.imgset->imgData()))[0]
		1: img = (*(self.imgset->imgData()))[slice]
		2: img = (*(self.imgset->imgData()))[slice, shot]
	end
	if( keyword_set( update ) ) then begin
		self.display->updateImg, img
	end else begin
		self.display->setImg, img
		self.display->setContrast, .5, 1.
		self.display->setGamma, 1.
		self.display->fitToScreen
	end

	if( self.dispdicom eq 1 ) then begin
		srcset = findDicomParent( self.imgset )
		if( obj_valid( srcset) ) then begin
			matrixSize = srcset->size( /n_dimensions )
			case matrixSize of
				0: dcm = (*(srcset->dicomData()))[0]
				1: dcm = (*(srcset->dicomData()))[slice]
				2: dcm = (*(srcset->dicomData()))[slice,shot]
			end
			self.dataflash->setDicomSource, dcm
			self.dataflash->display
		end else begin
			self.dataflash->hide
		end
	end else begin
		self.dataflash->hide
	end

	self.display->display

end


;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser::evalROIs
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imagebrowser::evalROIs

	roigroup = self.roiui->roiGroup()
	nrois    = roigroup->count()

		; if no rois defined we're already done

	if( nrois eq 0 ) then begin
		widget_control, self.idlist, set_value=''
		return
	end

		; if there's no image, we may return as well

	if( self.display->img() eq ptr_new() ) then begin
		widget_control, self.idlist, set_value=''
		return
	end

		; otherwise loop over all rois, calculate values and display
		; It may happen that a ROI is about to be defined, but has no
		; points yet - that's why we need to check for the number of
		; elements in text before setting.

	text = ''
	for i = 0, nrois-1 do begin
		roi   = roigroup->get( position=i )
		roino = roi->id()
		mask  = where( *(roi->filledMask()) gt 0, npixels)
		if( npixels eq 0 ) then continue
		meanval = mean( (*(self.display->img()))[mask] )
		minval  = min( (*(self.display->img()))[mask], max=maxval )
		if( roino eq 999 ) then begin
			text = [text,'Roi      : active']
		end else begin
			text = [text, string( roino, format='("Roi      :",i5)')]
		end
		text = [text, string( npixels, format='("# Pixels :",i5)')]
		if( n_elements(mask) gt 1 ) then begin
			text = [text, string( minval,  format='("Min      :",i5)')]
			text = [text, string( maxval,  format='("Max      :",i5)')]
			text = [text, string( meanval, format='("Mean     :",f7.1)')]
			stdval  = stddev( (*(self.display->img()))[mask] )
			text = [text, string( stdval,  format='("Stddev   :",f7.1)')]
		end else begin
			text = [text, string( meanval, format='("Value    :",f7.1)')]
		end
		text = [text,'']
	end
	if( n_elements(text) gt 1 ) then						$
		widget_control, self.idlist, set_value=text[1:*]	$
	else													$
		widget_control, self.idlist, set_value=''
end

;+=----------------------------------------------------------------------------
; Name  : mod_imagebrowser__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_imagebrowser__define
	obj = { MOD_IMAGEBROWSER   		   ,$
			inherits LUPEMOD   		   ,$
			roiui:			 obj_new() ,$
			display:		 obj_new() ,$
			contrast:		 obj_new() ,$
			zoom:			 obj_new() ,$
			cursorinfo:		 obj_new() ,$
			pan:			 obj_new() ,$
			dataflash:		 obj_new() ,$
			idvslider:				0L ,$
			idhslider:				0L ,$
			idclass:				0L ,$
			idname:					0L ,$
			iddicom:				0L ,$
			idinter:				0L ,$
			idlist:					0L ,$
			imgsetSelector:	 obj_new() ,$
			imgSet:			 obj_new() ,$
			dispdicom:				0  ,$
			currSlice:				0  ,$
			currShot:				0  ,$
			roiGroup:		 obj_new()  $ ; var for internal storage of one roi group (in between image changes)
		  }
end

