;+=----------------------------------------------------------------------------
; Name    : mod_imgArithmetics
; RCS     : $Id: mod_imgarithmetics__define.pro,v 1.4 2004/11/29 14:10:20 olivert Exp olivert $
; Desc    :
; Author  : (c) 2004 Oliver Thilmann
; TODO    : check whether the calculated result is a 2-dim image
;=------------------------------------------------------------------------------
function mod_imgArithmetics::revision
	rev = '$Revision: 1.4 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::init
; Desc  :
;=-----------------------------------------------------------------------------
function mod_imgArithmetics::init, g

    res = self->lupemod::init( 'mod_imgarithmetics', 'embedded', 0, g )
    if( res eq 0 ) then return, 0

		; set defaults

	self.formulas      = ['A-B','A-B > 0', 'abs(A-B)', 'a/maxa', 'A/B', 'A/B-1.', ' ', ' ', ' ', ' ']
	self.currSets      = ['', '', 'res']
	self.resizeFactor  = 1.
	self.sliceNo       = 0
	self.ipMethod      = 0
	self.lockSliders   = 1
	self.interpolate   = 0
	self.warnOverwrite = 1
	self.calcFullSet   = 1

	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::readyToExecute
; Syntax:
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_imgArithmetics::readyToExecute, option
	i18n = (*self.g).i18n
	sets = mrGetImageSet( (*self.g).imgSets, class='result', count=nsets )
	if( nsets eq 0 ) then begin
		x = dialog_message( i18n->get('err_noResultImageSet'), title=i18n->get('Info'), /info )
		return, 0
	end
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::map
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::map, option

	i18n = (*self.g).i18n

		; create the base widget - don't provide screen-size and title for an embedded widget!

	res = self->createBaseWidget( /row )
	if( res eq 0 ) then return

		; set up UI: three columns
		; bases in the columns are named b1 to bn, where 'n' is the hierarchy level

	bigbox = widget_base( self.baseid, /row, xpad=40, ypad=20 )
	c1 = widget_base( bigbox, /column ) 		; the A and B image sets
	c2 = widget_base( bigbox, /column ) 		; the result image
	c3 = widget_base( bigbox, /column )        ; rightmost column with all buttons

		; Operand image sets

	b1               = widget_base    ( c1, /column, /frame )
	b2               = widget_base    ( b1, /row )
	x                = widget_label   ( b2, value=i18n->get('ia_imageSetA'), scr_xsize=103 )
	self.idSets[0]   = widget_droplist( b2, value='', xsize=150, uvalue={o:self, m:'selectSetEvt'} )
	b2               = widget_base    ( b1, /row )
	self.display[0]  = obj_new        ('mrDisplay', b2, 256, 256, 256, 256 )
	self.idSlider[0] = widget_slider  ( b2, /vertical, scr_ysize=256, uvalue={o:self, m:'sliderEvt'} )

	x  = widget_label   ( c1, value='', ysize=10 )

	b1               = widget_base    ( c1, /column, /frame )
	b2               = widget_base    ( b1, /row )
	x                = widget_label   ( b2, value=i18n->get('ia_imageSetB'), scr_xsize=103 )
	self.idSets[1]   = widget_droplist( b2, value='', xsize=150, uvalue={o:self, m:'selectSetEvt'} )
	b2               = widget_base    ( b1, /row )
	self.display[1]  = obj_new        ('mrDisplay', b2, 256, 256, 256, 256 )
	self.idSlider[1] = widget_slider  ( b2, /vertical, scr_ysize=256, uvalue={o:self, m:'sliderEvt'} )

		; the result image set

	b1 = widget_base    ( c2, /column, /frame )

	b2               = widget_base    ( b1, /row )
	x                = widget_label   ( b2, value=i18n->get('ia_resultSetName'), scr_xsize=103 )
	self.idSets[2]   = widget_combobox( b2, /editable, value=' ', scr_xsize=150, uvalue={o:self, m:'selectResultSetEvt'} )
	x                = widget_label   ( b2, value=i18n->get('ia_hitReturnToSet') )
	b2               = widget_base    ( b1, /row )
	self.display[2]  = obj_new        ('mrDisplay', b2, 256, 256, 256, 256)
	self.contrast[2] = obj_new        ('mrContrast', b2, /vertical, ysize=256, xsize=55, colbars=0 )
	self.idSlider[2] = widget_slider  ( b2, /vertical, scr_ysize=256, uvalue={o:self, m:'sliderEvt'} )

	x  = widget_label   ( c2, value='', ysize=10 )

		; the formula block

	b1 = widget_base    ( c2, /column, /frame, xpad=10, ypad=9 )
	x  = widget_label   ( b1, value=i18n->get('ia_formula'), /align_left)
	bb                  = widget_base      ( b1, column=2, /frame )
	b2                  = widget_base      ( bb, /column )
	b3                  = widget_base      ( b2, /row )
	x                   = widget_label     ( b3, value=i18n->get('ia_formula')+':', scr_xsize=70 )
	x                   = widget_label     ( b3, value=i18n->get('ia_resultEquals'), scr_xsize=30 )
	self.idFormula      = widget_combobox  ( b3, /editable, value=self.formulas, scr_xsize=157, uvalue={o:self, m:'setFormulaEvt'} )
	self.idResizefactor = mrwidget_numfield( b2, /editable, value=self.resizeFactor, min=1, max=5, incr=.5,$
	                                         labeltext=i18n->get('ia_resizeFactor'), /float, /all_events, labelsize=210, fieldsize=30, format='(f3.1)', uvalue={o:self, m:'resizeFactorSelectEvt'} )
	b3                  = widget_base      ( b2 , /row )
	x                   = widget_label     ( b3, value=i18n->get('ia_interpolationMethod'), xsize=140 )
	self.idIpMethod     = widget_droplist  ( b3, value=[i18n->get('ia_nearestNeighbour'),i18n->get('ia_linear'),i18n->get('ia_cubic')], xsize=120, uvalue={o:self, m:'ipMethodSelectEvt'})
	widget_control, self.idIpMethod, set_droplist_select=self.ipMethod
	if( self.resizeFactor eq 1. ) then widget_control, self.idIpMethod, sensitive=0

	b2                  = widget_base      ( bb, /column )
	x                   = widget_button    ( b2, value=i18n->get('ia_calculate'), ysize=80, /frame, uvalue={o:self, m:'calculateButtonEvt'} )

	x  = widget_label   ( b1, value='', ysize=6 )

	x  = widget_label   ( b1, value=i18n->get('ia_info'), /align_left)
	b2 = widget_base    ( b1, /column, /frame, /base_align_left )
	b2 = widget_base    ( b2, /column, /base_align_left )
	x  = widget_label   ( b2, value=i18n->get('ia_info1')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info2')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info3')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info4')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info5')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info6')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info7')+' ' )
	x  = widget_label   ( b2, value=i18n->get('ia_info8')+' ' )

		; the options column

	c3 = widget_base  ( c3, /column, /frame, scr_ysize=614 )
	x  = widget_label ( c3, value=i18n->get('ia_title'), font='fixedsys', /align_center, /sunken_frame, frame=1, xsize=200)

	b1 = widget_base  ( c3, xsize=190, /align_center, /column )
	x  = widget_label ( b1, value='', ysize=20 )

	x  = widget_label ( b1, value=i18n->get('ia_mouse'), /align_left )
	b2 = widget_base  ( b1, /column, /frame, /base_align_left )
	;b3 = widget_base  ( b2, /row, ysize=30)
	;x  = widget_label ( b3, value=i18n->get('Button1')+':', xsize=60 )
	;x  = widget_label ( b3, value=i18n->get('ia_roiDefinition') )
	b3 = widget_base  ( b2, /row, ysize=30 )
	x  = widget_label ( b3, value=i18n->get('Button2')+':', xsize=60 )
	b4 = widget_base  ( b3, /exclusive, /row )
	x  = widget_button( b4, value=i18n->get('Zoom'), uvalue={o:self, m:'zoomModeEvt'} )
	y  = widget_button( b4, value=i18n->get('Pan'),  uvalue={o:self, m:'panModeEvt'} )
	widget_control, x, /set_button
	b3 = widget_base  ( b2, /row, ysize=30 )
	x  = widget_label ( b3, value=i18n->get('Button3')+':', xsize=60 )
	b4 = widget_base  ( b3, /exclusive, /row )
	x  = widget_button( b4, value=i18n->get('C/W'), uvalue={o:self, m:'contrastModeEvt'} )
	y  = widget_button( b4, value=i18n->get('Gamma'), uvalue={o:self, m:'gammaModeEvt'} )
	widget_control, x, /set_button

	x  = widget_label ( b1, value='', ysize=50 )

	x  = widget_label ( b1, value=i18n->get('Display')+':', /align_left )
	b2 = widget_base  ( b1, /column, /frame, /base_align_left )
	b3 = widget_base  ( b2, /row )
	x  = widget_label ( b3, value=i18n->get('ia_lockScrollbars'), xsize=140 )
	b4 = widget_base  ( b3, /row, /nonexclusive )
	x  = widget_button( b4, value='', uvalue={o:self, m:'lockSlidersButtonEvt'} )
	widget_control, x, set_button = self.lockSliders
	b3 = widget_base  ( b2, /row )
	x  = widget_label ( b3, value=i18n->get('Interpolate')+':', xsize=140 )
	b4 = widget_base  ( b3, /row, /nonexclusive )
	x  = widget_button( b4, value='', uvalue={o:self, m:'interpolateButtonEvt'} )
	widget_control, x, set_button=self.interpolate
	b3 = widget_base  ( b2, /row )
	x  = widget_button( b3, value=i18n->get('ia_applyAGeometryToAll'), xsize=160, uvalue={o:self, m:'applyGeometryEvt'} )
	b3 = widget_base  ( b2, /row )
	x  = widget_button( b3, value=i18n->get('ia_applyAContrastToAll'), xsize=160, uvalue={o:self, m:'applyContrastEvt'} )

	x  = widget_label ( b1, value='', ysize=50 )

	x  = widget_label ( b1, value=i18n->get('ia_calculation'), /align_left )
	b2 = widget_base  ( b1, /column, /frame, /base_align_left )
	b3 = widget_base  ( b2, /row )
	x  = widget_label ( b3, value=i18n->get('ia_warnOnOverwrite'), xsize=140 )
	b4 = widget_base  ( b3, /row, /nonexclusive )
	x  = widget_button( b4, value='', uvalue={o:self, m:'warnOverwriteButtonEvt'} )
	widget_control, x, set_button=self.warnOverwrite
	b3 = widget_base  ( b2, /exclusive, /column )
	x  = widget_button( b3, value=i18n->get('ia_calculateSingleImage'), uvalue={o:self, m:'calcSingleImageButtonEvt'} )
	y  = widget_button( b3, value=i18n->get('ia_calculateFullSet'),  uvalue={o:self, m:'calcFullSetButton'} )
	widget_control, x, set_button=1-self.calcFullSet
	widget_control, y, set_button=self.calcFullSet

		; close button

	x  = widget_label ( b1, value='', ysize=59)
	x  = widget_button( b1, value=i18n->get('CloseModule'), xsize=186, /frame, uvalue={o:self, m:'closeButtonEvt'} )

		; realize the widget and register with the xmanager

	self->placeAndRegister

		; create the remaining objects and let them register

	self.contrast[0] = obj_new('mrContrast')
	self.contrast[1] = obj_new('mrContrast')
	for i = 0, 2 do begin
		self.zoom[i]     = obj_new('mrZoom')
		self.pan[i]      = obj_new('mrPan')
		self.zoom[i]->selfregister,     self.display[i], button=2
		self.contrast[i]->selfregister, self.display[i], button=3
		self.display[i]->activate
	end

		; register for notifications and then update all fields

	(*self.g).imgSets->notify, self, 'update', ['add','remove']

	self->updateLists
	self->updateSliders
	self->display
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::unmap
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::unmap, _ref_extra=ex

	if( self.baseid eq 0L ) then return  ; uups - we don't exist :( Shouldn't happen

		; alternatively they could be all added to a container and the container destroyed...

	for i = 0, 2 do begin
		obj_destroy, self.zoom[i]
		obj_destroy, self.pan[i]
		obj_destroy, self.contrast[i]
		obj_destroy, self.display[i]
	end
	(*self.g).imgSets->notify, self, /remove

		; destroy the module - taken care off by the superclass

	self->lupemod::unmap, _strict_extra=ex
end


;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::selectSetEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::selectSetEvt, evt
	if( evt.id eq self.idSets[0] ) then id = 0 else id = 1
	widget_control, self.idSets[id], get_value=sets
	self.currSets[id] = sets[evt.index]
	self->updateSliders
	self->display, id
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::selectResultSetEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::selectResultSetEvt, evt
	self.currSets[2] = evt.str
	if( evt.index eq -1 ) then begin
		widget_control, self.idSets[2], get_value=sets
		if( where( sets eq evt.str ) eq -1 ) then begin
			widget_control, self.idSets[2], combobox_additem=evt.str, combobox_index=0
		end
	end else begin
		self->updateSliders
		self->display, 2
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::sliderEvt
; Desc  : If a slider is set to a value > slider_max, it sets itself to its
;         maximum value. This makes things convenient for us here (and elsewhere)
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::sliderEvt, evt

		; Sync other sliders if they're locked

	if( self.lockSliders ) then begin
		for i = 0, 2 do begin
			widget_control, self.idSlider[i], set_value=evt.value
		end
	end

		; We always redisplay all images. That's a bit of overhead, but
		; not a big one...

	self->display, /update   ; a bit of overhead, but what the heck
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::setFormulaEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::setFormulaEvt, evt
	if( evt.str eq '' ) then evt.str = ' '
	if( evt.index eq 0 )  then return ; user selected first in list ->nothing to do
	if( evt.index eq -1 ) then begin
		self.formulas[1:9]  = self.formulas[0:8]
		self.formulas[0]    = evt.str
	end else begin
		self.formulas[1:9]  = self.formulas[0:8]
		self.formulas[0]    = evt.str
	end
	widget_control, self.idFormula, set_value=self.formulas
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::resizeFactorSelectEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::resizeFactorSelectEvt, evt
	self.resizeFactor = evt.value
	if( self.resizeFactor eq 1. ) then begin
		widget_control, self.idIpMethod, sensitive=0
	end else begin
		widget_control, self.idIpMethod, sensitive=1
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::ipMethodSelectEvt
; Desc  : Calculate.
;         We do it here...
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::ipMethodSelectEvt, evt
	self.ipMethod = evt.index
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::calculateButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::calculateButtonEvt, evt
	self->calculate
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::zoomModeEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::zoomModeEvt, evt
	if( evt.select eq 1 ) then begin
		for i = 0, 2 do self.pan[i]->unregister
		for i = 0, 2 do self.zoom[i]->selfregister, self.display[i], button=2
	end else begin
		for i = 0, 2 do self.zoom[i]->unregister
		for i = 0, 2 do self.pan[i]->selfregister, self.display[i], button=2
	end
end
pro mod_imgArithmetics::panModeEvt, evt ; handled by above
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::contrastModeEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::contrastModeEvt, evt
	for i = 0, 2 do begin
		self.contrast[i]->setMode, 1-evt.select
	end
end
pro mod_imgArithmetics::gammaModeEvt, evt ; handled by above
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::lockSlidersButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::lockSlidersButtonEvt, evt
	self.lockSliders = evt.select
	if( self.locksliders ) then begin
		widget_control, self.idSlider[0], get_value=slice
		widget_control, self.idSlider[1], set_value=slice
		widget_control, self.idSlider[2], set_value=slice
		self->display, /update
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::interpolateButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::interpolateButtonEvt, evt
	self.interpolate = evt.select
	for i = 0, 2 do begin
		self.display[i]->setInterpolate, self.interpolate
		self.display[i]->display
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::applyGeometryEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::applyGeometryEvt, evt
	scaling  = self.display[0]->scaling()
	location = self.display[0]->location()
	print, location
	for i = 1, 2 do begin
		self.display[i]->setScaling, scaling
		self.display[i]->setLocation, location[0], location[1]
		self.display[i]->display
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::applyContrastEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::applyContrastEvt, evt
	contrast = self.display[0]->contrast()
	gamma    = self.display[0]->gamma()
	for i = 1, 2 do begin
		self.display[i]->setContrast, contrast[0], contrast[1]
		self.display[i]->setGamma, gamma
		self.display[i]->display
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::warnOverwriteButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::warnOverwriteButtonEvt, evt
	self.warnOverwrite = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::calcSingleImageButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::calcSingleImageButtonEvt, evt
end
pro mod_imgArithmetics::calcFullSetButton, evt
	self.calcFullSet = evt.select
end


;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::closeButtonEvt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::closeButtonEvt, evt
	self->unmap
end


;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::update, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::update, caller, evt

	currsets = strarr(3)
	currsets[*] = self.currSets
	self->updateLists
	if( self.baseid eq 0L ) then return    ; updateLists might have unmapped
	self->updateSliders

	for i = 0, 2 do begin
		if( currsets[i] ne self.currSets[i] ) then self->display, i
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::updateLists
; Desc  : Sets all image set selection droplists to valid values.
;         If possible, we try to keep the current selection for the droplists.
;         For the result combobox, it might be that the current result set
;         name is not in the list (because it does not exist, yet). In that
;         case, we add the name to the beginning of the list and make it
;         active (i.e. selected).
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::updateLists

	i18n = (*self.g).i18n

	allsets = mrGetImageSet( (*self.g).imgSets, class='result', count=nsets )
	if( nsets eq 0 ) then begin
		x = dialog_message([i18n->get('err_noResultImageSet'),i18n->get('err_closingModule')], title=i18n->get('Info'), /info )
		self->unmap
		return
	end

		; set the new namelist in the droplists - try to keep the current name

	names = strarr(nsets)
	for i = 0, nsets-1 do names[i] = allsets[i]->name()

	for i = 0, 1 do begin
		widget_control, self.idSets[i], set_value=names
		x = where( names eq self.currSets[i] )
		if( x eq -1 ) then x = 0
		widget_control, self.idSets[i], set_droplist_select=x
		self.currSets[i] = names[x]
	end

		; Set the list of names in the combobox. If the current resultSet
		; is an existing set, set the index correspondingly, otherwise
		; add the resultSet name to the beginning of the list

	x = where( names eq self.currSets[2] )
	if( x[0] eq -1 ) then begin
		names = [self.currSets[2], names]
		x = 0
	end
	widget_control, self.idSets[2], set_value = names
	widget_control, self.idSets[2], set_combobox_select = x[0]

end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::updateSliders
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::updateSliders

	for i = 0, 2 do begin
		set = mrGetImageSet( (*self.g).imgSets, class='result', name=self.currSets[i] )
		if( obj_valid(set) ) then begin
			nslices = set->nslices()
			widget_control, self.idSlider[i], set_slider_max=set->nslices()-1, /sensitive
			if( nslices eq 1 ) then widget_control, self.idSlider[i], sensitive=0
		end else begin
			widget_control, self.idSlider[i], set_slider_max=0, sensitive=0
		end
	end

end


;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::display
; Desc  : Display the currently selected image sets and slices
;         For the operand displays this is simple, as we know that the sets
;         exists and the slice is in the right range.
;         For the result set, we cannot be sure that the current result set
;         has already been calculated and thus exists. In that case we display
;         a black screen without further ado.
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::display, id, update=update

	if( n_elements(id) eq 0 ) then id = -1 ; 0 = (re-)display all
	update=keyword_set(update)

		; the operands

	for i = 0, 2 do begin
		if( id ne -1 and id ne i ) then continue
		set = mrGetImageSet( (*self.g).imgSets, class='result', name=self.currSets[i] )
		if( set ne obj_new() ) then begin        ; must be careful for result set
			widget_control, self.idSlider[i], get_value=slice
			self.display[i]->setImg, (*(set->imgData()))[slice], update=update
			if( not update ) then self.display[i]->fitToScreen
			self.display[i]->display
		end else begin
			self.display[i]->setImg, ptr_new( intarr(4,4), /no_copy ), /no_copy
			self.display[i]->fitToScreen
			self.display[i]->display
		end
	end

end


;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics::calculate
; Desc  : That's what we're here for
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics::calculate

	i18n = (*self.g).i18n

		; check whether a result name is set

	self.currSets[2] = strtrim( self.currSets[2], 2 )
	if( self.currSets[2] eq '' ) then begin
		x = dialog_message(i18n->get('ia_enterNameForResultSet'), title=i18n->get('Error'), /error )
		return
	end

		; check whether a formula has been entered/selected

	formula = strlowcase(strtrim( self.formulas[0], 2 ))
	if( formula eq '' ) then begin
		x = dialog_message(i18n->get('ia_enterFormula'), title=i18n->get('Error'), /error )
		return
	end

		; no '=' sign in the formula!

	if( strpos( formula, '=' ) ne -1 ) then begin
		x = dialog_message(i18n->get('ia_formulaNoEqualSign'), title=i18n->get('Error'), /error )
		return
	end

		; check whether the result set already exists and warn user, if wanted

	oldset = mrGetImageSet( (*self.g).imgSets, class='result', name=self.currSets[2], count=nsets )
	if( nsets eq 1 and self.warnOverwrite eq 1 ) then begin
		warnmessage = string( self.currSets[2], format=i18n->get('ia_warningSetExists') )
		res = mrdialog_select([warnmessage,i18n->get('ia_replaceIt')],[i18n->get('No'),i18n->get('Yes')],$
								group_leader=(*self.g).topid, /question, default=1, cancel=0, bsize=60, title=i18n->get('Question') )
		if( res eq 0 ) then return
	end

		; get operands

	setA = mrGetImageSet( (*self.g).imgSets, class='result', name=self.currSets[0] )
	setB = mrGetImageSet( (*self.g).imgSets, class='result', name=self.currSets[1] )

		; if both sets are used in the formula, they must be of identical image size

	if( stregex( formula, '(^|[^a-z])a([^a-z]|$)',/boolean ) and stregex( formula, '(^|[^a-z])b([^a-z]|$)',/boolean ) ) then begin
		if( ( setA->xsize() ne setB->xsize() ) or ( setA->ysize() ne setB->ysize() ) ) then begin
			x = dialog_message(i18n->get('ia_imageSetsMustBeSameSize'), title=i18n->get('Error'), /error)
			return
		end
	end

		; create a new image set

	nResSlices = 0
	if( stregex( formula, '(^|[^a-z])a([^a-z]|$)',/boolean ) ) then nResSlices = setA->nSlices()
	if( stregex( formula, '(^|[^a-z])b([^a-z]|$)',/boolean ) ) then nResSlices = setB->nSlices()

		; neither A nor B used in formula? We wouldn't know how big the result image should be!

	if( nResSlices eq 0 ) then begin
		x = dialog_message(i18n->get('ia_noInputSetsInFormula'), title=i18n->get('Error'), /error )
		return
	end

		; perform the calculation

	if( self.calcFullSet eq 1 ) then begin

			; find maximum values

		maxa = 0.
		maxb = 0.
		for i = 0, setA->nslices()-1 do begin
			maxa = max( *(*(setA->imgData()))[i] ) > maxa
		end
		for i = 0, setB->nslices()-1 do begin
			maxb = max( *(*(setB->imgData()))[i] ) > maxb
		end

			; create the set

		resSet = obj_new('mrImageSet', nResSlices, class='result', name=self.currSets[2] )

			; calculate all slices

		for i = 0, nResSlices-1 do begin
			a = float(*(*(setA->imgData()))[i])
			b = float(*(*(setB->imgData()))[i])
			res = execute( 'resultImg = '+formula )
			if( res eq 0 ) then begin
				x = dialog_message(i18n->get('ia_formulaError'), title=i18n->get('Error'), /error)
				obj_destroy, resSet
				return
			end

				; remove NANs

			x = where( finite(resultImg), complement=xx )
			if( xx[0] ne -1 ) then resultImg[xx] = 0.

			dims  = size( resultImg, /dimensions )
			if( self.resizeFactor ne 1. ) then begin
				newdims = fix( dims * self.resizeFactor )
				case self.ipmethod of
					0: resultImg =     congrid( temporary(resultImg), newdims[0], newdims[1], /center )
					1: resultImg = 0 > congrid( temporary(resultImg), newdims[0], newdims[1], /center, /interp )
					2: resultImg = 0 > congrid( temporary(resultImg), newdims[0], newdims[1],  cubic=-.5 )
				end
			end
			resSet->set, i, ptr_new( resultImg, /no_copy), /no_copy
		end

	end else begin

		resSet = obj_new('mrImageSet', 1, class='result', name=self.currSets[2] )
		widget_control, self.idSlider[0], get_value=sliceA
		widget_control, self.idSlider[1], get_value=sliceB
		a = float(*(*(setA->imgData()))[sliceA])
		b = float(*(*(setB->imgData()))[sliceB])
		maxa = max(a)
		maxb = max(b)
		res = execute( 'resultImg = '+formula )
		if( res eq 0 ) then begin
			x = dialog_message(i18n->get('ia_formulaError'), title=i18n->get('Error'), /error)
			obj_destroy, resSet
			return
		end

			; remove NANs

		x = where( finite(resultImg), complement=xx )
		if( xx[0] ne -1 ) then resultImg[xx] = 0.

		dims  = size( resultImg, /dimensions )
		if( self.resizeFactor ne 1. ) then begin
			newdims = fix( dims * self.resizeFactor )
			case self.ipmethod of
				0: resultImg =     congrid( temporary(resultImg), newdims[0], newdims[1], /center )
				1: resultImg = 0 > congrid( temporary(resultImg), newdims[0], newdims[1], /center, /interp )
				2: resultImg = 0 > congrid( temporary(resultImg), newdims[0], newdims[1],  cubic=-.5 )
			end
		end
		resSet->set, 0, ptr_new( resultImg, /no_copy), /no_copy

	end

		; If both sets have been used, parent is A, otherwise chose the set which has been
		; used in the formula

	if( stregex( formula, '(^|[^a-z])b([^a-z]|$)',/boolean ) ) then resSet->setParent, setB
	if( stregex( formula, '(^|[^a-z])a([^a-z]|$)',/boolean ) ) then resSet->setParent, setA

	if( obj_valid( oldset ) ) then begin
		(*self.g).imgSets->disableNotify
		(*self.g).imgSets->remove, oldset
		(*self.g).imgSets->enableNotify
	end
	(*self.g).imgSets->add, resSet   ; automatically updates via notification
	self->display, 0, /update
	self->display, 1, /update
	self->display, 2
end

;+=----------------------------------------------------------------------------
; Name  : mod_imgArithmetics__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_imgArithmetics__define
	obj = { MOD_IMGARITHMETICS 			,$
			inherits LUPEMOD   			,$
			display:		objarr(3)	,$
			zoom:			objarr(3)	,$
			pan:			objarr(3)	,$
			contrast:		objarr(3)	,$
			idSets:			lonarr(3)	,$
			idSlider:		lonarr(3)	,$
			idFormula:				0L	,$
			idResizeFactor:			0L	,$
			idIpMethod:				0L	,$
			formulas:		strarr(10)	,$
			currSets:		strarr(3)	,$ ; set '3' is the result set
			resizeFactor:			1.	,$
			ipMethod:				0	,$
			sliceNo:				0	,$
			lockSliders:			0	,$
			interpolate:			0	,$
			warnOverwrite:			0	,$
			calcFullSet:			1	 $
		  }
end
