;+=----------------------------------------------------------------------------
; Name    : mod_prefs
; RCS     : $Id: mod_prefs__define.pro,v 1.1 2004/11/11 11:34:51 olivert Exp olivert $
; Desc    : The 'preferences' window
;           As of know, all preferences are handled and dealt with here -
;           but in principle the modules should deal with their own
;           preferences.
;           Should have some interface class which deals with prefs and which
;           modules that want to publish preferences should implement.
;           Later. No need to shoot sparrows with cannons right now.
; Author  : (c) 2004 Oliver Thilmann
;=------------------------------------------------------------------------------
function mod_prefs::revision
	rev = '$Revision: 1.1 $'
	return, rev
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::init
; Syntax: obj = obj_new('lupemod', type, exclusive )
; Args  : <b>type</b>
;         One of 'noUI', 'top', 'embedded' (case-insensitive)
; Desc  :
;=-----------------------------------------------------------------------------
function mod_prefs::init, g

		; the preferences structure
		; in principle the calculation modules should save their own
		; preferences, but for the limited functionality to start with
		; it should be ok to store everything here and then we'll see
		; how it goes further down the road.
		; gotta get something working now

	prefs = {	lupe_prefs					 ,$
   				lang:			  obj_new()  ,$ ; a mrparameter object to hold the languages
   				logfile:				 ''  ,$ ; the log file
   				workingdir:				 ''	 ,$ ; the default working directory
   				floatingWins:			  0  ,$ ; have child windows float?
   				checkstudyUID:			  0	 ,$ ; data from only one studyUID in memory?
   				baseline_t0:			  0  ,$ ; lower edge for baseline averaging
   				baseline_t1:			  0  ,$ ; upper edge for baseline averaging
   				default_tr:		  		  0  ,$ ; default repetition time in ms
   				default_te:		  		  0  ,$ ; default echo time in ms
   				default_warn:			  0  ,$ ; flag a warning if default values are used
   				warnResultReplace:		  0  ,$ ; warn, if a result mrImageSet object should be replaced
   				autocalcctr:			  1  ,$ ; automatically calculate standard concentration maps?
   				implementationClassUID:	 ''  ,$ ; the implementation class UID for mrDicom
   				ssvd_sigthres:			  0. ,$ ; signal threshold for sSVD
   				ssvd_ctrthres:			  0. ,$ ; concentration threshold for sSVD
   				ssvd_matrixcutoff:		  0. ,$ ; matrix cutoff in sSVD
   				ssvd_cbvcutoff:			  0. ,$ ; cutoff value for CBV
   				ssvd_cbfcutoff:			  0. ,$ ; cutoff value for CBF
   				ssvd_mttcutoff:			  0.  $ ; cutoff value for MTT
	}

    res = self->lupemod::init( 'mod_prefs', 'top', 0, g )
    if( res eq 0 ) then return, 0

		; store preferences pointer in globals and return

	(*g).preferences = ptr_new( prefs, /no_copy )
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_prefs::readyToExecute
; Syntax:
; Args  :
; Desc  :
;=-----------------------------------------------------------------------------
function mod_prefs::readyToExecute, option
	return, 1
end


;+=----------------------------------------------------------------------------
; Name  : mod_prefs::map
; Syntax:
; Args  :
; Desc  : Creates the widget on screen
;=-----------------------------------------------------------------------------
pro mod_prefs::map, option

	prefs = *(*self.g).preferences
	i18n  = (*self.g).i18n

	res = self->createBaseWidget( /column, scr_xsize=345, scr_ysize=303, title='Lupe preferences' )
	if( res eq 0 ) then return

	tab  = widget_tab( self.baseid, uvalue={o:self, m:'dummyEvt'} )
	t1   = widget_base( tab, title=i18n->get('pref_general'),  /column, /frame, scr_xsize=322, scr_ysize=212 )
	t2   = widget_base( tab, title=i18n->get('pref_defaults'), /column, /frame, scr_xsize=322, scr_ysize=212 )
	t3   = widget_base( tab, title=i18n->get('pref_dicom'),    /column, /frame, scr_xsize=322, scr_ysize=212 )
	t4   = widget_base( tab, title=i18n->get('pref_svd'),      /column, /frame, scr_xsize=322, scr_ysize=212 )

		; GENERAL

	r = widget_label( t1, value='', scr_ysize=5 )
	r = widget_base( t1, /row )
	x = widget_label( r, value=i18n->get('pref_language'), scr_xsize=100 )
	x = widget_label( r, value='', scr_xsize=70 )
	curr = prefs.lang->get()
	val  = prefs.lang->validValues()
	vals = curr
	for i = 0, n_elements(val)-1 do if( val[i] ne curr ) then vals = [vals, val[i]]
	x = widget_droplist( r, value=vals, scr_xsize=130, uvalue={o:self, m:'languageSelectEvt'} )

	r = widget_base( t1, /row )
	x = widget_label( r, value=i18n->get('pref_defaultWorkingDir'), scr_xsize=100 )
	self.idgwdir = widget_text( r, value=prefs.workingdir, scr_xsize=180 )
	x            = widget_button( r, value='...', scr_xsize=20, uvalue={o:self, m:'wdirSelectEvt'} )

	r = widget_base( t1, /row )
	x = widget_label( r, value=i18n->get('pref_logfile'), scr_xsize=100 )
	self.idglog  = widget_text( r, value=prefs.logfile, scr_xsize=180 )
	x            = widget_button( r, value='...', scr_xsize=20, uvalue={o:self, m:'logfileSelectEvt'} )

	x = widget_base( t1, /nonexclusive )
	x = widget_button( x, value=i18n->get('pref_childWindowsFloat'), uvalue={o:self, m:'floatingWinsSelectEvt'} )
	widget_control, x, set_button=prefs.floatingWins

	x = widget_base( t1, /nonexclusive )
	self.idguid = widget_button( x, value=i18n->get('pref_onlyOneStudyUID'), uvalue={o:self, m:'checkStudyIDSelectEvt'} )
	widget_control, self.idguid, set_button=prefs.checkstudyUID

		; Basic default parameters for calculation

	r = widget_base( t2, /column )
	self.iddt0 = mrwidget_numfield( r, value=prefs.baseline_t0, labeltext=i18n->get('pref_baselineLowerBound'), labelsize=230, fieldsize=50, /integer, /editable, min=0, uvalue={o:self, m:'baselinet0SelectEvt'} )
	self.iddt1 = mrwidget_numfield( r, value=prefs.baseline_t1, labeltext=i18n->get('pref_baselineUpperBound'), labelsize=230, fieldsize=50, /integer, /editable, min=1, uvalue={o:self, m:'baselinet1SelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.default_tr, labeltext=i18n->get('pref_defaultTR'), labelsize=230, fieldsize=50, /integer, /editable, min=100, incr=100, uvalue={o:self, m:'defaultTRSelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.default_te, labeltext=i18n->get('pref_defaultTE'), labelsize=230, fieldsize=50, /integer, /editable, min=0, max=300, uvalue={o:self, m:'defaultTESelectEvt'} )
	x = widget_base( r, /nonexclusive )
	x = widget_button( x, value=i18n->get('pref_warnDefaultValues'), uvalue={o:self, m:'warnDefaultUsageSelectEvt'} )
	widget_control, x, set_button=prefs.default_warn
	x = widget_base( r, /nonexclusive )
	x = widget_button( x, value=i18n->get('pref_warnReplaceResult'), uvalue={o:self, m:'warnResultReplaceSelectEvt'} )
	widget_control, x, set_button=prefs.warnResultReplace
	x = widget_base( r, /nonexclusive )
	x = widget_button( x, value=i18n->get('pref_autocalcCtr'), uvalue={o:self, m:'autocalcCtrSelectEvt'} )
	widget_control, x, set_button=prefs.autocalcctr

		; DICOM

	r = widget_base( t3, /column )
	x = widget_label( r, value='', scr_ysize=5 )
	x = widget_base ( r, /row )
	y = widget_label( x, value=i18n->get('pref_implementationClassUID'), scr_xsize=140 )
	self.idicUID = widget_text( x, value=prefs.implementationClassUID, scr_xsize=140, /editable, uvalue={o:self, m:'icUIDChangeEvt'}, /all_events )
	x = widget_label( r, value='', scr_ysize=5 )
	x = widget_button( r, value=i18n->get('pref_getFromDicomFile'), scr_xsize=20, uvalue={o:self, m:'icUIDButtonEvt'} )

		; SVD

	r = widget_base( t4, /column )
	x = mrwidget_numfield( r, value=prefs.ssvd_sigthres,     labeltext=i18n->get('pref_thresholdSignal'), labelsize=230, fieldsize=50, /integer, /editable, min=0, uvalue={o:self, m:'sigthresSelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.ssvd_ctrthres,     labeltext=i18n->get('pref_thresholdCtr'), labelsize=230, fieldsize=50, /float, /editable, min=0., format='(f6.0)', incr=10., uvalue={o:self, m:'ctrthresSelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.ssvd_matrixcutoff, labeltext=i18n->get('pref_wMatrixCutoff'), labelsize=230, fieldsize=50, /float, /editable, min=0., max=1., incr=0.01, format='(f4.2)', uvalue={o:self, m:'wcutoffSelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.ssvd_cbvcutoff,    labeltext=i18n->get('pref_CBVcutoff'), labelsize=230, fieldsize=50, /float, /editable, min=0., max=9000., incr=100., format='(f5.0)', uvalue={o:self, m:'cbvcutoffSelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.ssvd_cbfcutoff,    labeltext=i18n->get('pref_CBFcutoff'), labelsize=230, fieldsize=50, /float, /editable, min=0., max=9000., incr=100., format='(f5.0)', uvalue={o:self, m:'cbfcutoffSelectEvt'} )
	x = mrwidget_numfield( r, value=prefs.ssvd_mttcutoff,    labeltext=i18n->get('pref_MTTcutoff'), labelsize=230, fieldsize=50, /float, /editable, min=0., max=9000., incr=100., format='(f5.0)', uvalue={o:self, m:'mttcutoffSelectEvt'} )

		; Buttons

	b = widget_base  ( self.baseid, column=3, /align_right )
	x = widget_button( b, value=i18n->get('Save'), scr_xsize=80, /frame, uvalue={o:self, m:'saveButtonEvt'})
	x = widget_label ( b, value='  ' )
	x = widget_button( b, value=i18n->get('Reset'), scr_xsize=80, /frame, uvalue={o:self, m:'resetButtonEvt'})
	x = widget_label ( b, value='  ' )
	x = widget_button( b, value='Ok', scr_xsize=80, /frame, uvalue={o:self, m:'okButtonEvt'} )
	self->placeAndRegister
end



;==============================================================================
;
;                           E V E N T   H A N D L I N G
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::saveButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::saveButtonEvt, evt
	i18n = (*self.g).i18n
	res = self->save()
	if( res eq 0 ) then begin
		x = dialog_message(i18n->get('pref_failedToSave'), title=i18n->get('Error'), /error )
	end else begin
		x = dialog_message(i18n->get('pref_successSave'), title=i18n->get('Info'), /info )
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::resetButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::resetButtonEvt, evt
	i18n = (*self.g).i18n
	res = mrdialog_select(i18n->get('pref_resetToDefault'), [i18n->get('No'),i18n->get('Yes')], title=i18n->get('Question'), $
						  group_leader=(*self.g).topid, /question, default=1, cancel=0, bsize=60 )
	if( res eq 1 ) then begin
		self->setDefaults
		self->unmap
		self->map
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::okButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::okButtonEvt, evt
	self->unmap
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::languageSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::languageSelectEvt, evt
	prefs = (*self.g).preferences
	widget_control, evt.id, get_value=lang
	(*prefs).lang->set, lang[evt.index]
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::wdirSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::wdirSelectEvt, evt
	i18n = (*self.g).i18n
	prefs = (*self.g).preferences
	file  = dialog_pickfile( path=(*prefs).workingdir, get_path=newpath, title=i18n->get('pref_selectDirectory'), /directory )
	if( file eq '' ) then return
	(*self.g).workingdir = newpath
	(*prefs).workingdir  = newpath
	widget_control, self.idgwdir, set_value=newpath
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::logfileSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::logfileSelectEvt, evt
	i18n = (*self.g).i18n
	prefs = (*self.g).preferences
	file  = dialog_pickfile( path=(*self.g).workingdir, get_path=newpath, title=i18n->get('pref_selectLogFile') )
	if( file eq '' ) then return
	widget_control, self.idglog, set_value=file
	(*prefs).logfile = file
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::floatingWinsSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::floatingWinsSelectEvt, evt
	(*(*self.g).preferences).floatingWins = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::autocalcctrSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::autocalcctrSelectEvt, evt
	(*(*self.g).preferences).autocalcctr = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::checkStudyIDSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::checkStudyIDSelectEvt, evt
	prefs = (*self.g).preferences
	i18n  = (*self.g).i18n
	if( evt.select eq 0 ) then begin
		res = mrdialog_select([i18n->get('pref_checkStudyUID1'), i18n->get('pref_checkStudyUID2')],$
						      [i18n->get('Yes'), i18n->get('No')], default_button=1, bsize=80, /warning, group_leader=self.baseid, title=i18n->get('Warning') )
		if( res eq 1 ) then begin
			widget_control, self.idguid, set_button=1
			(*prefs).checkstudyUID = 1 ; should already be '1'
		end else begin
			(*prefs).checkstudyUID = 0
		end
	end else begin
		if( (*self.g).imgSets->count() gt 0 ) then begin
			res = dialog_message([i18n->get('pref_cannotSwitchOn1'), i18n->get('pref_cannotSwitchOn2')], title=i18n->get('Error'), /error)
			widget_control, self.idguid, set_button=0
			(*prefs).checkstudyUID = 0;
		end else begin
			(*prefs).checkstudyUID = 1
		end
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::baselinet0SelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::baselinet0SelectEvt, evt
	prefs = (*self.g).preferences
	widget_control, self.iddt0, get_value=value
	if( value ge (*prefs).baseline_t1 ) then begin
		(*prefs).baseline_t1 = value+1
		widget_control, self.iddt1, set_value=value+1
	end
	(*prefs).baseline_t0 = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::baselinet1SelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::baselinet1SelectEvt, evt
	prefs = (*self.g).preferences
	widget_control, self.iddt1, get_value=value
	if( value le (*prefs).baseline_t0 ) then begin
		if( value eq 0 ) then value = 1
		(*prefs).baseline_t0 = value-1
		widget_control, self.iddt0, set_value=(*prefs).baseline_t0
		widget_control, self.iddt1, set_value=value
	end
	(*prefs).baseline_t1 = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::defaultTRSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::defaultTRSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).default_tr = fix(value)
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::defaultTESelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::defaultTESelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).default_te = fix(value)
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::warnDefaultUsageSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::warnDefaultUsageSelectEvt, evt
	(*(*self.g).preferences).default_warn = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::warnResultReplaceSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::warnResultReplaceSelectEvt, evt
	(*(*self.g).preferences).warnResultReplace = evt.select
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::icUIDButtonEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::icUIDButtonEvt, evt
	i18n  = (*self.g).i18n
	prefs = (*self.g).preferences

	file = dialog_pickfile( path=(*self.g).workingdir, title=i18n->get('pref_selectDicomFile') )
	if( file eq '' ) then return
	dicom = obj_new('idlffdicom')
	res = dicom->read( file )
	if( res eq 0 ) then begin
		x = dialog_message(i18n->get('err_openFile'), title=i18n->get('Error'), /error)
		return
	end

		; in our case, the value from the dicom file contained one '\0' character
		; at the end of the string, which messed up the rest.
		; The strsplit function helps getting rid of this superfluous '\0'

    valptr = dicom->getValue('0002'x,'0012'x)
	if( n_elements(valPtr) eq 1 ) then begin
		x = strsplit( *valptr[0], /extract )
		(*prefs).implementationClassUID = x[0]
		widget_control, self.idicuid, set_value = x[0]
	end else begin
		x = dialog_message(i18n->get('pref_errReadingImplClassUID'), title=i18n->get('Error'), /error)
	end
    obj_destroy, dicom
    mrdicom_setUIDs, 'mrimg', (*prefs).implementationClassUID

end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::icUIDChangeEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::icUIDChangeEvt, evt
	widget_control, self.idicuid, get_value=uid
	(*(*self.g).preferences).implementationClassUID = uid
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::sigthresSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::sigthresSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).ssvd_sigthres = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::ctrthresSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::ctrthresSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).ssvd_ctrthres = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::wcutoffSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::wcutoffSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).ssvd_matrixcutoff = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::cbvcutoffSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::cbvcutoffSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).ssvd_cbvcutoff = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::cbfcutoffSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::cbfcutoffSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).ssvd_cbfcutoff = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::mttcutoffSelectEvt, evt
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs::mttcutoffSelectEvt, evt
	widget_control, evt.id, get_value=value
	(*(*self.g).preferences).ssvd_mttcutoff = value
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::dummyEvt, evt
; Desc  : needed to catch events from tab changes
;=-----------------------------------------------------------------------------
pro mod_prefs::dummyEvt, evt
end

;==============================================================================
;
;                                 I N T E R N A L
;
;==============================================================================


;+=----------------------------------------------------------------------------
; Name  : mod_prefs::setDefaults
; Desc  : Sets default values for all preference variables - to be used to
;         create a preferences file from scratch
;=-----------------------------------------------------------------------------
pro mod_prefs::setDefaults
	prefs = (*self.g).preferences
	if( obj_valid( (*prefs).lang ) ) then obj_destroy, (*prefs).lang
	(*prefs).lang = obj_new('mrparameter', 'lang', 'string', 1 )
	(*prefs).lang->set, 'English'
	(*prefs).lang->setValidValues, ['English','Deutsch']
	(*prefs).workingdir = (*self.g).homedir
	(*prefs).logfile = (*self.g).homedir+path_sep()+'lupe.log'
	(*prefs).floatingWins = 1
	(*prefs).checkstudyUID = 1
	(*prefs).baseline_t0 = 3
	(*prefs).baseline_t1 = 10
	(*prefs).default_tr = 1500
	(*prefs).default_te = 29
	(*prefs).default_warn = 1
	(*prefs).warnResultReplace = 1
	(*prefs).autocalcctr = 1
	(*prefs).implementationClassUID = '0.0.00.0.0000.0.0'
	(*prefs).ssvd_sigthres = 110
	(*prefs).ssvd_ctrthres = 150.
	(*prefs).ssvd_matrixcutoff = 0.15
	(*prefs).ssvd_cbvcutoff = 1000.
	(*prefs).ssvd_cbfcutoff = 2000.
	(*prefs).ssvd_mttcutoff = 4000.
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::load
; Desc  : Load preferences from preferences.xml via IDLffXMLSAX mod_prefs_xml
;=-----------------------------------------------------------------------------
function mod_prefs::load
	catch, error_status
	if( error_status ne 0 ) then begin
		catch, /cancel
		help, /last_message
		close, /all, /force      ; this does not work for XML files and is a known
		return, 0                ; bug in IDL - check the newsgroup for more info
	end
	xmlparser = obj_new('MOD_PREFS_XML', (*self.g).preferences )
	if( xmlparser eq obj_new() ) then return, 0
	filename = (*self.g).homedir + '/preferences.xml'
	xmlparser->parsefile, filename
	obj_destroy, xmlparser
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs::save
; Desc  : Save the preferences to preferences.xml
;=-----------------------------------------------------------------------------
function mod_prefs::save
	prefs = (*self.g).preferences
	filename = (*self.g).homedir + '/preferences.xml'
	openw, unit, filename, /get_lun, error=err
	if( err ne 0 ) then begin
		message, !ERROR_STATE.MSG
	end
	printf, unit, '<?xml version="1.0"?>'
	printf, unit, '<lupe_preferences>
	printf, unit, '  <languages>
	valid = (*prefs).lang->validValues()
	for i = 0, n_elements(valid)-1 do printf, unit, '    <available>'+valid[i]+'</available>'
	printf, unit, '    <current>'+(*prefs).lang->get()+'</current>'
	printf, unit, '  </languages>
	printf, unit, '  <program_defaults>
	printf, unit, '    <workingdir>'+(*prefs).workingdir+'</workingdir>'
	printf, unit, '    <logfile>'+(*prefs).logfile+'</logfile>'
	printf, unit, '    <floatingWins>'+strtrim((*prefs).floatingWins,2)+'</floatingWins>
	printf, unit, '    <checkstudyUID>'+strtrim((*prefs).checkstudyUID,2)+'</checkstudyUID>'
	printf, unit, '  </program_defaults>
	printf, unit, '  <calc_defaults>'
	printf, unit, '    <t0>'+strtrim((*prefs).baseline_t0,2)+'</t0>'
	printf, unit, '    <t1>'+strtrim((*prefs).baseline_t1,2)+'</t1>'
	printf, unit, '    <TR>'+strtrim((*prefs).default_tr,2)+'</TR>'
	printf, unit, '    <TE>'+strtrim((*prefs).default_te,2)+'</TE>'
	printf, unit, '    <warnDefaultUse>'+strtrim((*prefs).default_warn,2)+'</warnDefaultUse>'
	printf, unit, '    <warnResultReplace>'+strtrim((*prefs).warnResultReplace,2)+'</warnResultReplace>'
	printf, unit, '    <autocalcctr>'+strtrim((*prefs).autocalcctr,2)+'</autocalcctr>'
	printf, unit, '  </calc_defaults>'
	printf, unit, '  <dicom>'
	printf, unit, '    <implementationClassUID>'+string((*prefs).implementationClassUID)+'</implementationClassUID>'
	printf, unit, '  </dicom>'
	printf, unit, '  <SVD>
	printf, unit, '    <sigthres>'+strtrim((*prefs).ssvd_sigthres,2)+'</sigthres>'
	printf, unit, '    <ctrthres>'+strtrim((*prefs).ssvd_ctrthres,2)+'</ctrthres>'
	printf, unit, '    <matrixcutoff>'+strtrim((*prefs).ssvd_matrixcutoff,2)+'</matrixcutoff>'
	printf, unit, '    <cbvcutoff>'+strtrim((*prefs).ssvd_cbvcutoff,2)+'</cbvcutoff>'
	printf, unit, '    <cbfcutoff>'+strtrim((*prefs).ssvd_cbfcutoff,2)+'</cbfcutoff>'
	printf, unit, '    <mttcutoff>'+strtrim((*prefs).ssvd_mttcutoff,2)+'</mttcutoff>'
	printf, unit, '  </SVD>
	printf, unit, '</lupe_preferences>
	free_lun, unit
	return, 1
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs__define
; Desc  : Private Procedure (constructor)
;=-----------------------------------------------------------------------------
pro mod_prefs__define
	obj = { MOD_PREFS		   		,$
			inherits LUPEMOD   		,$
			idglog:				0L	,$
			idgwdir:			0L	,$
			idguid:				0L	,$
			iddt0:				0L	,$
			iddt1:				0L	,$
			idicuid:			0L	 $
		  }
end


;==============================================================================
;
;                                  XML READER
;
;==============================================================================

;+=----------------------------------------------------------------------------
; Name  : mod_prefs_xml::init
; Desc  : The parser reads the xml file and stores the read values in the
;         prefs structure
;=-----------------------------------------------------------------------------
function mod_prefs_xml::init, prefs
	self.maj   = -1
	self.prefs = prefs
	return, self->IDLffXMLSAX::init()
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs_xml::startElement
; Desc  : We have only and always two hierarchy levels which simplifies things
;         a lot.
;=-----------------------------------------------------------------------------
pro mod_prefs_xml::startElement, URI, local, strName, attr, value
	case local of
		'lupe_preferences': self.maj = -1
		'languages'       : begin
			if( (*self.prefs).lang eq obj_new() ) then obj_destroy, (*self.prefs).lang
			(*self.prefs).lang = obj_new('mrparameter', 'lang', 'string', 1)
			self.maj = 1
		end
		'available'				: self.min = 1
		'current'				: self.min = 2
		'program_defaults'		: self.maj = 2
		'workingdir'			: self.min = 1
		'logfile'				: self.min = 2
		'floatingWins'			: self.min = 3
		'checkstudyUID'			: self.min = 4
		'calc_defaults' 		: self.maj = 3
		't0'					: self.min = 1
		't1'					: self.min = 2
		'TR'					: self.min = 3
		'TE'					: self.min = 4
		'warnDefaultUse'		: self.min = 5
		'warnResultReplace' 	: self.min = 6
		'autocalcctr'			: self.min = 7
		'dicom'             	: self.maj = 4
		'implementationClassUID': self.min = 1
		'SVD'					: self.maj = 5
		'sigthres'				: self.min = 1
		'ctrthres'				: self.min = 2
		'matrixcutoff'			: self.min = 3
		'cbvcutoff'				: self.min = 4
		'cbfcutoff'				: self.min = 5
		'mttcutoff'				: self.min = 6
		else					: print, 'unknown element'
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs_xml::endElement
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs_xml::endElement, URI, local, strName
	case local of
		'lupe_preferences': self.maj = -1
		else			  : self.min = 0
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs_xml::characters
; Desc  :
;=-----------------------------------------------------------------------------
pro mod_prefs_xml::characters, data
	case self.maj of
	1: begin							; language
		case self.min of
		0:
		1: (*self.prefs).lang->setValidValues, data
		2: (*self.prefs).lang->set, data
		end
	end
	2: begin							; program defaults
		case self.min of
		0:
		1: (*self.prefs).workingdir    = data
		2: (*self.prefs).logfile       = data
		3: (*self.prefs).floatingWins  = fix(data)
		4: (*self.prefs).checkstudyUID = fix(data)
		end
	end
	3: begin							; calculation defaults
		case self.min of
		0:
		1: (*self.prefs).baseline_t0       = fix(data)
		2: (*self.prefs).baseline_t1       = fix(data)
		3: (*self.prefs).default_tr        = float(data)
		4: (*self.prefs).default_te        = float(data)
		5: (*self.prefs).default_warn      = fix(data)
		6: (*self.prefs).warnResultReplace = fix(data)
		7: (*self.prefs).autocalcctr       = fix(data)
		end
	end
	4: begin							; Dicom
		case self.min of
		0:
		1: (*self.prefs).implementationClassUID = data
		end
	end
	5: begin							; SVD
		case self.min of
		0:
		1: (*self.prefs).ssvd_sigthres     = float(data)
		2: (*self.prefs).ssvd_ctrthres     = float(data)
		3: (*self.prefs).ssvd_matrixcutoff = float(data)
		4: (*self.prefs).ssvd_cbvcutoff    = float(data)
		5: (*self.prefs).ssvd_cbfcutoff    = float(data)
		6: (*self.prefs).ssvd_mttcutoff    = float(data)
		else: print, 'Unknown self.min ' + string(self.min) + ' ' + data
		end
	end
	else:
	end
end

;+=----------------------------------------------------------------------------
; Name  : mod_prefs_xml__define
; Desc  : create separate object - dunno if multiple inheritance works under
;         IDL, but that's cleaner anyway
;=-----------------------------------------------------------------------------
pro mod_prefs_xml__define
	obj = { MOD_PREFS_XML, $
			inherits IDLffXMLSAX		,$
			prefs:			 ptr_new()  ,$ ; pointer to the preferences structure
			maj:                      0	,$ ; the current element
			min:                      0  $ ; the current subelement
		}
end